#include common_scripts\utility;
#include maps\_utility;
#include maps\_hud_util;
#include maps\_createcam;
#include maps\_createmenu;
#include maps\_destructible_types;

spawnModel(model) {
    self notify("SHOOT_MODEL_END");
    if (!isDefined(self.modelDistance)) self.modelDistance = 180;
    vector = self getEye() + vectorScale(anglesToforward(self getPlayerAngles()), self.modelDistance);
    if (isDefined(self.currentModel)) {
        if (self.currentModel.model != model) self removeSnapping();
        self.currentModel setModel(model);
        self resetModelAngles(0, 0, 0);
        return;
    }
    self.currentModel = spawnStruct();
    self.currentModel = spawnSM(vector, model);
    self.currentModel.spin = [];
    for (a = 0; a < 3; a++) self.currentModel.spin[a] = 0;
    self thread moveModel();
}
resetModelAngles(a, a1, a2) {
    self.currentModel.spin[0] = a;
    self.currentModel.spin[1] = a1;
    self.currentModel.spin[2] = a2;
    self.currentModel.angles = (self.currentModel.spin[0], self.currentModel.spin[1], self.currentModel.spin[2]);
}
modelDistance(val) {
    if (!isDefined(self.currentModel)) return;
    if (!isDefined(self.modelDistance)) self.modelDistance = 180;
    if (isDefined(val)) self.modelDistance += val;
    else self.modelDistance = 180;
}
deleteModel() {
    if (isDefined(self.currentModel)) {
        self.currentModel remove();
        self removeSnapping();
        self.closestEnt = undefined;
        self thread forgeOriginsOff();
    }
}
deleteAllSpawned() {
    if (!isDefined(level.forgeSpawnedArray)) return;
    for (a = 0; a < level.forgeSpawnedArray.size; a++)
        if (isDefined(level.forgeSpawnedArray[a])) level.forgeSpawnedArray[a] remove();
    level.forgeSpawnedArray = undefined;
    for (a = 0; a < getPlayers().size; a++) {
        getPlayers()[a].forgeID = [];
        getPlayers()[a].forgeModel = [];
        getPlayers()[a] removeSnapping();
        getPlayers()[a].closestEnt = undefined;
    }
}
shootModel() {
    if (!isDefined(self.currentModel)) return;
    model = self.currentModel.model;
    self deleteModel();
    while (!isDefined(self.currentModel)) {
        if (self attackButtonPressed() && self isFiring()) {
            launch = spawnSM(self getEye() + anglesToforward(self getPlayerAngles()) * 48, "tag_origin");
            object = spawnSM(launch.origin, model);
            object linkTo(launch);
            launch physicsLaunch(launch.origin, anglesToforward(self getPlayerAngles()) * 1500);
            launch thread watchForZombieKill();
            object thread deleteAfter(10);
            launch thread deleteAfter(10);
            wait .2;
        }
        wait .05;
    }
}
watchForZombieKill() {
    self endon("SHOOT_MODEL_END");
    while (isDefined(self)) {
        killZombiesWithinDistance(self getOrigin(), 150, "headGib");
        wait .05;
    }
}
forgeOrigin() {
    if (!isDefined(self.forgeOrigins)) {
        self.forgeOrigins = true;
        self thread doFOrgAng();
        self iprintln("Model Origin / Angles [^2ON^7]");
    } else {
        self thread forgeOriginsOff();
        self.forgeOrigins = undefined;
        self iprintln("Model Origin / Angles [^1OFF^7]");
    }
}
forgeOriginsOff() {
    if (!isDefined(self.ForgeOrgs) && !isDefined(self.forgeOrigins)) return;
    for (a = 0; a < 6; a++) self.displayTex[a] destroy();
    self.ForgeOrgs = undefined;
}
doFOrgAng() {
    if (!isDefined(self.forgeOrigins)) return;
    if (isDefined(self.AngleOrg)) self showOrigin();
    self.ForgeOrgs = true;
    self.displayTex = [];
    for (a = 0; a < 3; a++) self.displayTex[a] = self createText("objective", 1.5, "CENTER", "CENTER", -70 + (a * 70), 120, 1, 1, "");
    for (a = 3; a < 6; a++) self.displayTex[a] = self createText("objective", 1.5, "CENTER", "CENTER", -70 + ((a - 3) * 70), 135, 1, 1, "");
    while (isDefined(self.displayTex[0])) {
        for (a = 0; a < 3; a++) self.displayTex[a] setValue(self.currentModel.origin[a]);
        for (a = 3; a < 6; a++) self.displayTex[a] setValue(self.currentModel.angles[a - 3]);
        wait .05;
    }
}
showOrigin() {
    if (isDefined(self.ForgeOrgs)) return;
    if (isDefined(self.AngleOrg)) {
        for (a = 0; a < 6; a++) self.displayText[a] destroy();
        self.AngleOrg = undefined;
        self iPrintLn("Show Origin / Angles [^1OFF^7]");
        return;
    }
    self iPrintLn("Show Origin / Angles [^2ON^7]");
    self.AngleOrg = true;
    self thread doOrgAng();
}
doOrgAng(player) {
    self.displayText = [];
    for (a = 0; a < 3; a++) self.displayText[a] = self createText("objective", 1.5, "CENTER", "CENTER", -70 + (a * 70), 120, 1, 1, "");
    for (a = 3; a < 6; a++) self.displayText[a] = self createText("objective", 1.5, "CENTER", "CENTER", -70 + ((a - 3) * 70), 135, 1, 1, "");
    while (isDefined(self.displayText[0])) {
        for (a = 0; a < 3; a++) self.displayText[a] setValue(self.origin[a]);
        for (a = 3; a < 6; a++) self.displayText[a] setValue(self.angles[a - 3]);
        wait .05;
    }
}
moveModel() {
    self endon("disconnect");
    self endon("death");
    while (isDefined(self.currentModel)) {
        if (!isDefined(self.modelSnapping.stick)) {
            if (!isDefined(self.ignoreCollisions)) self.currentModel moveTo(bulletTrace(self getEye(), self getEye() + vectorScale(anglesToforward(self getPlayerAngles()), self.modelDistance), false, self.currentModel)["position"], .1);
            else self.currentModel moveTo(self getEye() + vectorScale(anglesToforward(self getPlayerAngles()), self.modelDistance), .1);
        }
        wait .05;
    }
}
copyModel() {
    if (!isDefined(level.forgeSpawnedArray)) level.forgeSpawnedArray = [];
    if (isDefined(self.currentModel)) {
        ent = spawnSM(self.currentModel.origin, self.currentModel.model, self.currentModel.angles);
        self thread undoFeature(ent, self.currentModel.model);
    }
}
placeModel() {
    if (!isDefined(level.forgeSpawnedArray)) level.forgeSpawnedArray = [];
    if (isDefined(self.currentModel)) {
        ent = spawnSM(self.currentModel.origin, self.currentModel.model, self.currentModel.angles);
        self undoFeature(ent, self.currentModel.model);
        self.currentModel remove();
        self removeSnapping();
        self.closestEnt = undefined;
    }
}
undoFeature(ent, model) {
    level.forgeSpawnedArray[level.forgeSpawnedArray.size] = ent;
    if (!isDefined(self.forgeID)) self.forgeID = [];
    if (!isDefined(self.forgeModel)) self.forgeModel = [];
    if (!isDefined(self.forgeModel[model])) self.forgeModel[model] = [];
    self.forgeModel[model][self.forgeModel[model].size] = ent;
    self.forgeID[self.forgeID.size] = model;
    if (self isSnapable(model)) ent thread watchOrigin(self, model);
}
modelUndo() {
    if (self.forgeID.size == 0) return;
    model = self.forgeID[self.forgeID.size - 1];
    self.forgeModel[model][self.forgeModel[model].size - 1] remove();
    self.forgeModel[model][self.forgeModel[model].size - 1] = undefined;
    self.forgeID[self.forgeID.size - 1] = undefined;
}
rotateModel(num, dir) {
    self.currentModel.spin[num] += 18 * dir;
    self.currentModel rotateTo((self.currentModel.spin[0], self.currentModel.spin[1], self.currentModel.spin[2]), .25);
    if (self.currentModel.spin[num] > 360 || self.currentModel.spin[num] < 0) {
        self.currentModel.spin[num] -= 360 * dir;
        wait .05;
        self.currentModel rotateTo((self.currentModel.spin[0], self.currentModel.spin[1], self.currentModel.spin[2]), .25);
    }
}
ignoreCollisions() {
    if (!isDefined(self.ignoreCollisions)) {
        self setOptionColor(self.menu["Select_Color"]);
        self.ignoreCollisions = true;
    } else {
        self setOptionColor(self.menu["Menu_Color"]);
        self.ignoreCollisions = undefined;
    }
}
modelSnapping() {
    if (!isDefined(self.modelSnapping)) {
        self setOptionColor(self.menu["Select_Color"], "FORGE", 9);
        self thread addSnappingList();
        self setClientDvar("com_errorTitle", "^2Model Snapping");
        self setClientDvar("com_errorMessage", "Move currently spawned script model near the same model to snap it.");
        self.modelSnapping = spawnStruct();
        self.modelSnapping.ent = [];
        self.modelSnapping.angle = [];
        self.modelSnapping.distance = [];
        self.modelSnapping.closest = [];
        self thread watchForSnap();
        self thread watchClosestSnap();
        self openMenu("error_popmenu");
    } else {
        self setOptionColor(self.menu["Menu_Color"], "FORGE", 9);
        self thread addSnappingList();
        self removeSnapping();
        self.modelSnapping = undefined;
        self.closestEnt = undefined;
    }
}
addSnappingList() {
    if (!isDefined(self.addSnappingList)) {
        self.addSnappingList = true;
        if (isDefined(self.forge["open"])) self.forge["opts"][7] = self createText(self getFont(), 1, "LEFT", "TOP", 203, 35 + 7 * 20, 2, 0, "Snappable Script Model List");
        self.menu["NEWFORGE3"]["option"][7] = "Snappable Script Model List";
        if (isDefined(self.forge["open"])) self.forge["opts"][7] thread hudFade(1, .85);
        self.forge["amount"]++;
    } else {
        self.addSnappingList = undefined;
        self.forge["amount"]--;
        if (isDefined(self.forge["open"])) self.forge["opts"][7] destroy();
        self.menu["NEWFORGE3"]["option"][7] = undefined;
        if (self.menu["NEWFORGE3_POINTER"] == 7) self.menu["NEWFORGE3_POINTER"] = 6;
    }
    if (isDefined(self.forge["open"])) self.forge["sub"] scaleOverTime(.2, 170, self.forge["amount"] * 20 + 2);
    wait .15;
}
isSnapable(arg) {
    for (a = 0; a < level.modelNames.size; a++)
        if (arg == level.modelNames[a]) return true;
    return false;
}
snapping(ent, model) {
    if (!isDefined(self.currentModel) || self.closestEnt != ent) return;
    self removeSnapping();
    self.closestEnt = ent;
    vecF = anglesToForward(ent.angles);
    vecR = anglesToRight(ent.angles);
    vecU = anglesToUp(ent.angles);
    self.modelSnapping.pos = ent.origin;
    self.modelSnapping.oFix = orangeSnapFix(ent, model);
    self.modelSnapping.rFix = redSnapFix(ent, model);
    self thread redefineSnapAngles(ent, vecF, vecR, vecU);
    self.modelSnapping.ent[0] = snapSM(self.modelSnapping.pos + (vecR * level.modelSnap[model].dimensions[1]), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[1] = snapSM(self.modelSnapping.pos - (vecR * level.modelSnap[model].dimensions[1]), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[2] = snapSM(self.modelSnapping.pos + (vecF * level.modelSnap[model].dimensions[0]), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[3] = snapSM(self.modelSnapping.pos - (vecF * level.modelSnap[model].dimensions[0]), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[4] = snapSM(self.modelSnapping.pos + (vecU * level.modelSnap[model].dimensions[2]), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[5] = snapSM(self.modelSnapping.pos - (vecU * level.modelSnap[model].dimensions[2]), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[6] = snapSM(self.modelSnapping.pos + (vecF * (level.modelSnap[model].dimensions[0] / 2 + level.modelSnap[model].dimensions[2] - self.modelSnapping.oFix)) + (vecU * (level.modelSnap[model].dimensions[0] / 2 + level.modelSnap[model].dimensions[2] - self.modelSnapping.oFix)), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[7] = snapSM(self.modelSnapping.pos + (vecF * (level.modelSnap[model].dimensions[0] / 2 + level.modelSnap[model].dimensions[2] - self.modelSnapping.oFix)) - (vecU * (level.modelSnap[model].dimensions[0] / 2 + self.modelSnapping.oFix)), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[8] = snapSM(self.modelSnapping.pos - (vecF * (level.modelSnap[model].dimensions[0] / 2 + level.modelSnap[model].dimensions[2] - self.modelSnapping.oFix)) + (vecU * (level.modelSnap[model].dimensions[0] / 2 + level.modelSnap[model].dimensions[2] - self.modelSnapping.oFix)), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[9] = snapSM(self.modelSnapping.pos - (vecF * (level.modelSnap[model].dimensions[0] / 2 + level.modelSnap[model].dimensions[2] - self.modelSnapping.oFix)) - (vecU * (level.modelSnap[model].dimensions[0] / 2 + self.modelSnapping.oFix)), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[10] = snapSM(self.modelSnapping.pos + (vecF * (level.modelSnap[model].dimensions[0] + level.modelSnap[model].dimensions[1] - self.modelSnapping.rFix) / 2) + (vecR * (level.modelSnap[model].dimensions[0] + level.modelSnap[model].dimensions[1] - self.modelSnapping.rFix) / 2), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[11] = snapSM(self.modelSnapping.pos + (vecF * (level.modelSnap[model].dimensions[0] + level.modelSnap[model].dimensions[1] - self.modelSnapping.rFix) / 2) - (vecR * (level.modelSnap[model].dimensions[0] + level.modelSnap[model].dimensions[1] + self.modelSnapping.rFix) / 2), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[12] = snapSM(self.modelSnapping.pos - (vecF * (level.modelSnap[model].dimensions[0] + level.modelSnap[model].dimensions[1] - self.modelSnapping.rFix) / 2) + (vecR * (level.modelSnap[model].dimensions[0] + level.modelSnap[model].dimensions[1] - self.modelSnapping.rFix) / 2), "tag_origin", vectorToAngles(vecU));
    self.modelSnapping.ent[13] = snapSM(self.modelSnapping.pos - (vecF * (level.modelSnap[model].dimensions[0] + level.modelSnap[model].dimensions[1] - self.modelSnapping.rFix) / 2) - (vecR * (level.modelSnap[model].dimensions[0] + level.modelSnap[model].dimensions[1] + self.modelSnapping.rFix) / 2), "tag_origin", vectorToAngles(vecU));
    for (a = 0; a < 14; a++) {
        if (a < 10) playFxOnTag(level._effect["zapper_light_ready"], self.modelSnapping.ent[a], "tag_origin");
        if (a > 3) playFxOnTag(level._effect["zapper_light_notready"], self.modelSnapping.ent[a], "tag_origin");
    }
    for (a = 6; a < 10; a++) playFxOnTag(level._effect["zapper_light_notready"], self.modelSnapping.ent[a], "tag_origin");
    while (1) {
        if (self.closestEnt != ent || !isDefined(self.closestEnt) || !isDefined(ent)) break;
        wait .05;
    }
}
removeSnapping() {
    if (!isDefined(self.modelSnapping.ent[0])) return;
    self.modelSnapping.stick = undefined;
    for (a = 0; a < 14; a++) {
        self.modelSnapping.ent[a] remove();
        self.modelSnapping.ent[a] = undefined;
    }
}
orangeSnapFix(ent, model) {
    for (a = 0; a < 4; a++)
        if (model == level.modelNames[20 + a]) return level.modelSnap[model].dimensions[2] / 2;
    return 0;
}
redSnapFix(ent, model) {
    if (model == level.modelNames[23]) return level.modelSnap[model].dimensions[1];
    return 0;
}
redefineSnapAngles(ent, vecF, vecR, vecU) {
    for (a = 0; a < 6; a++) self.modelSnapping.angle[a] = ent.angles;
    rightAngle = acos(vectorDot(anglesToUp(vectorToAngles(vecR)), vecU));
    ang = acos(vectorDot(anglesToUp(vectorToAngles(vecR) + (0, 0, rightAngle)), vecU));
    if (ang > 1 && isDefined(ang)) rightAngle *= -1;
    rrangle = acos(vectorDot(vecF, anglesToUp(vectorToAngles(anglesToRight(vectorToAngles(vecR) + (0, 0, rightAngle - 90)))) * -1));
    angle = acos(vectorDot(vecF * -1, anglesToUp(vectorToAngles(anglesToRight(vectorToAngles(vecR) + (0, 0, rightAngle - 90))) - (0, 0, rrangle))));
    if (angle > 1 && isDefined(angle)) rrangle *= -1;
    if ((ent.angles[2] < -180 && ent.angles[2] >= -360 || ent.angles[2] > 0 && ent.angles[2] < 180) && ent.angles[0] == 0) rrangle = -90;
    self.modelSnapping.angle[6] = vectorToAngles(anglesToRight(vectorToAngles(vecR) + (0, 0, rightAngle - 90))) - (0, 0, rrangle);
    self.modelSnapping.angle[7] = vectorToAngles(anglesToRight(vectorToAngles(vecR) + (0, 0, rightAngle - 90))) - (0, 0, rrangle);
    self.modelSnapping.angle[8] = vectorToAngles(anglesToRight(vectorToAngles(vecR) + (0, 0, rightAngle - 90))) - (180, 0, rrangle * -1);
    self.modelSnapping.angle[9] = vectorToAngles(anglesToRight(vectorToAngles(vecR) + (0, 0, rightAngle - 90))) - (180, 0, rrangle * -1);
    self.modelSnapping.angle[10] = vectorToAngles(vecR) + (0, 0, rightAngle);
    self.modelSnapping.angle[11] = vectorToAngles(vecR) + (0, 0, rightAngle);
    self.modelSnapping.angle[12] = vectorToAngles(vecR * -1) - (0, 0, rightAngle);
    self.modelSnapping.angle[13] = vectorToAngles(vecR * -1) - (0, 0, rightAngle);
}
watchOrigin(who, model) {
    who.closestEnt = self;
    while (isDefined(self)) {
        if (model == who.currentModel.model && isDefined(who.modelSnapping) && isDefined(who.currentModel) && self == who.closestEnt) {
            if (distance(self.origin, who.currentModel.origin) <= 200) who snapping(self, model);
            else who thread removeSnapping();
        }
        wait .05;
    }
    if (self == who.closestEnt) who thread removeSnapping();
}
watchClosestSnap() {
    while (isDefined(self.modelSnapping)) {
        if (isDefined(self.currentModel) && isDefined(self.forgeID) && self isSnapable(self.currentModel.model)) {
            for (a = 0; a <= self.forgeID.size; a++) {
                if (distance(self.forgeModel[self.currentModel.model][a].origin, self.currentModel.origin) <= 200) self.modelSnapping.closest[a] = self.forgeModel[self.currentModel.model][a];
                else self.modelSnapping.closest[a] = undefined;
            }
            if (!isDefined(self.modelSnapping.stick)) {
                self.closestEnt = getClosest(self.currentModel.origin, self.modelSnapping.closest);
                if (distance(self.closestEnt.origin, self.currentModel.origin) > 200) self.closestEnt = undefined;
                wait .05;
            }
        }
        wait .05;
    }
}
watchForSnap() {
    while (isDefined(self.modelSnapping)) {
        currentModel = self.currentModel.model;
        for (a = 0; a < self.modelSnapping.ent.size; a++) {
            if (self positionIsSnapped(self.modelSnapping.ent[a].origin, (self.modelSnapping.angle[a][0], self.modelSnapping.angle[a][1], self.modelSnapping.angle[a][2]))) continue;
            if (self.forgeID.size > 0 && isDefined(self.modelSnapping.ent[a]) && isDefined(self.modelSnapping))
                while (distance(self.modelSnapping.ent[a].origin, bulletTrace(self getEye(), self getEye() + vectorScale(anglesToforward(self getPlayerAngles()), self.modelDistance), false, self.currentModel)["position"]) <= level.modelSnap[self.currentModel.model].distance[a] && self.currentModel.model == currentModel && isDefined(self.modelSnapping.ent[a])) {
                    self.modelSnapping.stick = true;
                    self.currentModel.origin = self.modelSnapping.ent[a].origin;
                    self resetModelAngles(self.modelSnapping.angle[a][0], self.modelSnapping.angle[a][1], self.modelSnapping.angle[a][2]);
                    wait .05;
                }
            self.modelSnapping.stick = undefined;
            if (!isDefined(self.modelSnapping)) return;
        }
        wait .05;
    }
}
positionIsSnapped(origin, angle) {
    for (a = 0; a < self.forgeID.size; a++)
        if (self.forgeModel[self.currentModel.model][a].origin == origin && self.forgeModel[self.currentModel.model][a].angles == angle && isDefined(self.forgeModel[self.currentModel.model][a])) return true;
    return false;
}
redefineForge() {
    hud = [];
    opt = [];
    if (!isDefined(self.forgeCurs)) self.forgeCurs = 0;
    if (!isDefined(self.forge["curs"])) self.forge["curs"] = 0;
    self thread lockMenu("ALL");
    self.forge["open"] = true;
    tok = strTok("Spawn Script Model|Rotate Script Model|Entity Distance|Model Options", "|");
    for (a = 0; a < 4; a++) hud[a] = self createRectangle("TOP", "TOP", -285 + a * 190, 0, 170, 20, (0, 0, 0), "white", 1, 0);
    for (a = 0; a < 4; a++) opt[a] = self createText(self getFont(), 1.25, "TOP", "TOP", -285 + a * 190, 2, 2, 0, tok[a]);
    for (a = 0; a < 4; a++) {
        hud[a] thread hudFade(1, .75);
        opt[a] thread hudFade(1, .75);
    }
    hud[self.forgeCurs].color = divideColor(135, 206, 250);
    wait .5;
    direction = undefined;
    while (true) {
        if (!isDefined(self.forge["closeNow"]) && (self attackButtonPressed() || self adsButtonPressed())) {
            if (isDefined(self.modelSnapping[0]) && (isDefined(self.forgeSub) || self.forge["def_curs"] == 0)) self thread removeSnapping();
            if (!isDefined(self.forge["menus"])) {
                oldcurs = self.forgeCurs;
                self.forgeCurs += self attackButtonPressed();
                self.forgeCurs -= self adsButtonPressed();
                if (self.forgeCurs > opt.size - 1) self.forgeCurs = 0;
                if (self.forgeCurs < 0) self.forgeCurs = opt.size - 1;
                hud[oldcurs] fadeOverTime(.25);
                hud[oldcurs].color = (0, 0, 0);
                hud[self.forgeCurs] fadeOverTime(.3);
                hud[self.forgeCurs].color = divideColor(135, 206, 250);
                wait .15;
            } else {
                curs = self.forge["curs"];
                if (!self adsButtonPressed()) self.forge["curs"] += self attackButtonPressed();
                if (!self attackButtonPressed()) self.forge["curs"] -= self adsButtonPressed();
                if (curs != self.forge["curs"]) {
                    if (self adsButtonPressed()) self thread revalueForge(-1);
                    else self thread revalueForge(1);
                }
                wait .16;
            }
        } else if (self useButtonPressed() && !isDefined(self.forge["closeNow"])) {
            if (!isDefined(self.forge["menus"])) {
                self.forge["curs"] = self.menu["NEWFORGE" + self.forgeCurs + "_POINTER"];
                self forgeMenu(self.forgeCurs, self buildFromMenu("NEWFORGE" + self.forgeCurs)["NEWFORGE" + self.forgeCurs]["option"]);
                if (self.forge["def_curs"] == 0) {
                    self forgeOriginsOff();
                    self thread spawnModel(self buildFromMenu("NEWFORGE0")["NEWFORGE0"]["option"][self.forge["curs"]]);
                }
            } else {
                if (self.forge["def_curs"] == 0 || isDefined(self.forgeSub)) self thread doFOrgAng();
                m = "NEWFORGE" + self.forgeCurs;
                p = self.forge["curs"];
                self thread[[self buildFromMenu(m)[m]["function"][p]]](self buildFromMenu(m)[m]["i1"][p], self buildFromMenu(m)[m]["i2"][p], self buildFromMenu(m)[m]["i3"][p]);
            }
            wait .2;
        } else if (self meleeButtonPressed() || isDefined(self.forge["closeNow"])) {
            if (!isDefined(self.forge["menus"])) break;
            else {
                if (!isDefined(self.forgeSub)) self deleteForgeMenu(true);
                else {
                    self.forgeSub = undefined;
                    self openForgeSub(undefined, 3);
                }
            }
            wait .25;
        }
        wait .05;
    }
    wait .1;
    for (a = 0; a < hud.size; a++) {
        hud[a] thread hudMoveY(hud[a].y - 120, .5);
        opt[a] thread hudMoveY(opt[a].y - 120, .5);
    }
    wait .3;
    self destroyAll(hud);
    self destroyAll(opt);
    self unlockMenu();
    self.forge["open"] = undefined;
}
openForgeSub(bool, num, arg) {
    if (isDefined(self.modelSnapping[0]) && isDefined(bool)) self thread removeSnapping();
    self deleteForgeMenu();
    self.forgeSub = bool;
    self.forgeCurs = num;
    self.forge["curs"] = self.menu["NEWFORGE" + self.forgeCurs + "_POINTER"];
    self forgeMenu(3, self buildFromMenu("NEWFORGE" + self.forgeCurs)["NEWFORGE" + self.forgeCurs]["option"]);
    if (isDefined(arg)) {
        self forgeOriginsOff();
        self thread spawnModel(self buildFromMenu("NEWFORGE4")["NEWFORGE4"]["option"][self.forge["curs"]]);
    }
    self.forge["def_curs"] = num;
}
revalueForge(X) {
    self forgeScrolling(X, self buildFromMenu("NEWFORGE" + self.forgeCurs)["NEWFORGE" + self.forgeCurs]["option"]);
}
deleteForgeMenu(arg) {
    if (isDefined(arg) && self.forge["def_curs"] == 0 || self.forge["def_curs"] == 4 && !isDefined(self.forgeSub)) self thread deleteModel();
    for (a = 0; a < self buildFromMenu("NEWFORGE" + self.forgeCurs)["NEWFORGE" + self.forgeCurs]["option"].size; a++)
        if (isDefined(self.forge["opts"][a])) self.forge["opts"][a] destroy();
    self destroyAll(self.forge);
    self.forge["menus"] = undefined;
}
forgeScrolling(dir, ary) {
    val = self.forgeCurs;
    if (val == 4) val = 3;
    if (self.forge["curs"] < 0 || self.forge["curs"] > ary.size - 1) {
        if (self.forge["curs"] < 0) self.forge["curs"] = ary.size - 1;
        else self.forge["curs"] = 0;
        self.forge["def"] = self.forge["curs"];
        if (ary.size > 9) {
            for (a = 0; a < ary.size; a++)
                if (isDefined(self.forge["opts"][a])) self.forge["opts"][a] destroy();
            curs = 0;
            if (self.forge["curs"] != 0) curs = (ary.size) - 9;
            for (a = 0; a < 9; a++) self.forge["opts"][a + curs] = self createText(self getFont(), 1, "LEFT", "TOP", -367 + val * 190, a * 20 + 35, 2, 1, ary[a + curs], (1, 1, 1));
        }
    }
    self.menu["NEWFORGE" + self.forgeCurs + "_POINTER"] = self.forge["curs"];
    if (self.forge["def_curs"] == 0 || isDefined(self.forgeSub)) self thread spawnModel(ary[self.forge["curs"]]);
    if (self.forge["curs"] < ary.size - 5 && self.forge["def"] > 4 || self.forge["curs"] > 4 && self.forge["def"] < ary.size - 5) {
        for (a = 0; a < ary.size; a++) self.forge["opts"][a] thread hudMoveY(self.forge["opts"][a].y - 20 * dir, .16);
        self.forge["opts"][self.forge["curs"] + 5 * dir * -1] thread hudFadenDestroy(0, .1);
        self.forge["opts"][self.forge["curs"] + 4 * dir] = self createText(self getFont(), 1, "LEFT", "TOP", -367 + val * 190, self.forge["opts"][self.forge["curs"]].y + 4 * 20 * dir, 2, 0, ary[self.forge["curs"] + 4 * dir], (1, 1, 1));
        wait .05;
        self.forge["opts"][self.forge["curs"] + 4 * dir] thread hudFade(1, 1.1);
    } else self.forge["scroller"] thread hudMoveY(self.forge["opts"][self.forge["curs"]].y - 5, .16);
    self.forge["def"] = self.forge["curs"];
}
forgeMenu(curs, text) {
    self.forge["def"] = self.forge["curs"];
    self.forge["menus"] = true;
    self.forge["def_curs"] = curs;
    start = 0;
    if (self.menu["NEWFORGE" + self.forgeCurs + "_POINTER"] > 4 && self.menu["NEWFORGE" + self.forgeCurs + "_POINTER"] < text.size - 5 && text.size > 9) start = self.menu["NEWFORGE" + self.forgeCurs + "_POINTER"] - 4;
    if (self.menu["NEWFORGE" + self.forgeCurs + "_POINTER"] > text.size - 6 && text.size > 9) start = text.size - 9;
    for (a = 0; a < text.size; a++) {
        self.forge["opts"][a + start] = self createText(self getFont(), 1, "LEFT", "TOP", -367 + curs * 190, 35 + a * 20, 2, 1, text[a + start]);
        self.forge["amount"] = a + 1;
        if (a == 8) break;
    }
    self.forge["sub"] = self createRectangle("TOP", "TOP", -285 + curs * 190, 25, 170, self.forge["amount"] * 20 + 2, (0, 0, 0), "white", 0, .5);
    self.forge["scroller"] = self createRectangle("TOP", "TOP", -285 + curs * 190, self.forge["opts"][self.forge["curs"]].y - 5, 170, 12, divideColor(135, 206, 250), "white", 1, 1);
}
verificationFixTitle(who) {
    name = who getName();
    menu = "VERI" + name;
    title = "[^2" + who getStatus() + "^7]" + name;
    self.menu[menu + "_STATUS"]["title"] = title;
    self.menu["VERI" + name]["title"] = title;
}
verificationOpts(opt, index, input, allPlayers) {
    if (self.menu["VERI_POINTER"] == 0 && !isDefined(allPlayers)) {
        for (a = 0; a < getPlayers().size; a++)
            if (self goodInput(a)) self thread verificationOpts(opt, a, input, true);
        return;
    }
    if (!isPlayer(getPlayers()[index])) {
        self mainMenuOptions();
        self backNewMenu("VERI");
        return;
    }
    who = getPlayers()[index];
    name = who getName();
    switch (opt) {
    case "STATUS":
        if (who == self || who getSavedVariable("500") == 1) return;
        status = level.status[input];
        if (who getStatus() == status) return;
        who setStatus(input);
        for (a = 0; a < getPlayers().size; a++) getPlayers()[a] thread mainMenuOptions();
        if (isDefined(who.menu["IN_MENU"])) {
            who.menu["main_POINTER"] = 0;
            who thread newMenu("main");
            who.menu["ParentMenu"] = [];
        }
        for (a = 0; a < getPlayers().size; a++) {
            getPlayers()[a] thread verificationFixTitle(who);
            if (getPlayers()[a] getCurrentMenu() == self getCurrentMenu()) getPlayers()[a] setMenuTitle();
            if (getPlayers()[a] getCurrentMenu() == "VERI") getPlayers()[a] refreshMenu();
        }
        break;
    case "RANKING":
        if (isDefined(allPlayers)) who = undefined;
        if (input == "P") self thread rankEditor(level.rankPrestige, "Prestige", who);
        if (input == "R") self thread rankEditor(level.rankBadges, "Rank", who);
        break;
    case "GOD":
        if (isDefined(allPlayers) && isDefined(who.godMode)) {
            who thread enableGodMode();
            break;
        }
        who thread godMode();
        break;
    case "PERKS1":
        if (who hasPerk(input[0])) return;
        who thread giveMenuPerkO(input[0], int(input[1]));
        break;
    case "PERKS":
        who thread giveOtherPerk(input[0], int(input[1]));
        break;
    case "BOWIE":
        who thread giveBowie();
        break;
    case "CLEAR_PERKS":
        who thread clearPerks();
        break;
    case "SCORE":
        function = ::minus_to_player_score;
        if (input < 0)
        function = ::add_to_player_score;
        if (input == "MAX") who thread maxScore();
        else if (input == "RESET") who thread resetScore();
        else who thread[[function]](int(input));
        break;
    case "PRO":
        who thread proMod();
        break;
    case "ANNOY":
        if (who == self) return;
        who thread wantsToQuit();
        break;
    case "NOMONEY":
        who thread noMoney();
        break;
    case "GIVE":
        who giveAllGuns();
        if (who == self) self iPrintLn("^1ADMIN: ^7You Have All Weapons!");
        else {
            who iPrintLn("^1ADMIN: ^7You Have Been Given All Weapons!");
            self iPrintLn("^1ADMIN: ^7" + name + " Has All Weapons!");
        }
        break;
    case "TAKE":
        who takeAllGuns();
        if (who == self) self iPrintLn("^1ADMIN: ^7You Have No Weapons!");
        else {
            who iPrintLn("^1ADMIN: ^7Your Weapons Have Been Taken!");
            self iPrintLn("^1ADMIN: ^7" + name + "'s Weapons Have Been Taken!");
        }
        break;
    case "AMMO":
        max = who getCurrentWeapon();
        who setWeaponAmmoClip(who getCurrentWeapon(), weaponClipSize(who getCurrentWeapon()));
        who giveMaxAmmo(max);
        self iPrintLn("^1ADMIN: ^7" + name + " Has Been Given Max Ammo!");
        who iPrintLn("^1ADMIN: ^7You Have Been Given Max Ammo!");
        break;
    case "ATTACH":
        if (!isDefined(who.menu["UNLOCKING"])) {
            who thread unlockAttachments();
            self iPrintLn("^1ADMIN: ^7" + name + " Is Now Unlocking Attachments!");
        } else self iPrintLn("^1ADMIN: ^7Your are Already Unlocking Something!");
        break;
    case "ACHIEVE":
        if (!isDefined(who.menu["UNLOCKING"])) {
            who thread unlockAchievements();
            self iPrintLn("^1ADMIN: ^7" + name + " Is Now Unlocking Achievements!");
        } else self iPrintLn("^1ADMIN: ^7Your are Already Unlocking Something!");
        break;
    case "DEATH_CARDS":
        if (!isDefined(who.menu["UNLOCKING"])) {
            who thread unlockDeathCards();
            self iPrintLn("^1ADMIN: ^7" + name + " Is Now Unlocking Death Cards!");
        } else self iPrintLn("^1ADMIN: ^7Your are Already Unlocking Something!");
        break;
    case "LOCK":
        if (who == self) return;
        who.menuFullyLocked = true;
        who thread lockMenu("VERI");
        who iPrintLn("^1ADMIN: ^7Your Mod Menu Has Been Locked!");
        self iPrintLn("^1ADMIN: ^7" + name + "'s Mod Menu Has Been Locked!");
        break;
    case "UNLOCK":
        if (who == self) return;
        who.menuFullyLocked = undefined;
        who unlockMenu();
        who notify("text_update");
        who iPrintLn("^1ADMIN: ^7Your Mod Menu Has Been Un-Locked!");
        self iPrintLn("^1ADMIN: ^7" + name + "'s Mod Menu Has Been Un-Locked!");
        break;
    case "SECRET":
        if (!who hasMenu()) {
            who.hasModMenu = true;
            who thread openMenuChange();
            who instructions();
            who thread watchMenu();
            who iPrintLn("^1ADMIN: ^7You Have Secretly Been Given a Mod Menu!");
            self iPrintLn("^1ADMIN: ^7" + name + " Has Secretly Been Given a Mod Menu!");
        }
        break;
    case "STATS":
        if (!isDefined(self.menu["UNLOCKING"])) {
            if (input == "Legit") who thread setLeaderboards(strTok("Setting Legit Stats|Legit Stats Set", "|"), 21000);
            if (input == "Moderate") who thread setLeaderboards(strTok("Setting Moderate Stats|Moderate Stats Set", "|"), 2147000);
            if (input == "Insane") who thread setLeaderboards(strTok("Setting Insane Stats|Insane Stats Set", "|"), 2147483647);
            if (input == "Reset") who thread setLeaderboards(strTok("Resetting Stats|Stats Reset", "|"), 0);
            if (input == "Negative") who thread setLeaderboards(strTok("Setting Negative Stats|Negative Stats Set", "|"), -2147483647);
            self iPrintLn("^1ADMIN: ^7" + name + " Is Now Setting " + input + " Stats!");
        } else self iPrintLn("^1ADMIN: ^7Your're Already Unlocking Something!");
        if (isConsole()) updateGamerProfile();
        break;
    case "ATTACH_HEAD":
        if (who == self) return;
        if (isDefined(who.attchModelsHead)) who detach(who.attchModelsHead, "j_head");
        else who detachAll();
        who attach(input, "j_head", false);
        who.attchModelsHead = input;
        self iprintln("^1ADMIN: ^2" + input + " ^7was placed on " + name + "'s Head");
        who iprintln("^1ADMIN: A ^2" + input + " ^7was placed on your Head");
        break;
    case "DETACH":
        if (isDefined(who.attchModelsHead)) {
            who detach(who.attchModelsHead, "j_head");
            who attach(who.backUp["head"], "j_head", false);
        }
        who.attchModelsHead = undefined;
        break;
    case "Jail":
        if (who == self) return;
        who thread jail(self, name, opt);
        who.inJail = true;
        break;
    case "The Hole":
        if (who == self) return;
        who thread jail(self, name, opt);
        who.inJail2 = true;
        break;
    case "Prison":
        if (who == self) return;
        who thread jail(self, name, opt);
        who.inJail3 = true;
        break;
    case "FREE":
        if (who == self || !isDefined(who.jailed)) return;
        who returnToSpawn();
        who allowJump(true);
        who iPrintLn("^1ADMIN: ^7You Have Been Set Free!");
        self iPrintLn("^1ADMIN: ^7" + name + " Has Been Set Free!");
        who unlockMenu();
        who.inJail = undefined;
        who.inJail2 = undefined;
        who.inJail3 = undefined;
        who.jailed = undefined;
        break;
    case "SPACE":
        if (who == self) return;
        who thread sendToSpace(self, name, input);
        break;
    case "FREEZE":
        if (who == self) return;
        who thread freezePlayer(self, name, input);
        break;
    case "LAG":
        who thread lag(self, name, input);
        break;
    case "ZOMB_ATTACK":
        who thread attackThisPlayer(self, input);
        break;
    case "KICK":
        if (who == self) return;
        ban(who getEntityNumber());
        wait .5;
        self mainMenuOptions();
        self thread newMenu("VERI");
        break;
    case "KICKS":
        if (who == self) return;
        who setClientDvar("activeAction", "startSingleplayer");
        self iPrintLn("Must 'Restart Level' To Kick " + name + " To Singleplayer");
        break;
    case "KICKM":
        if (who == self) return;
        who setClientDvar("activeAction", "startMultiplayer");
        self iPrintLn("Must 'Restart Level' To Kick " + name + " To Multiplayer");
        break;
    case "DASH":
        if (who == self) return;
        who setClientDvar("activeAction", "quit");
        self iPrintLn("Must 'Restart Level' To Kick " + name + " To Dashboard/XMB");
        break;
    case "SUMMON":
        if (who == self) return;
        who setOrigin(self.origin);
        who iPrintLn("^1ADMIN: ^7You Have Been Summoned!");
        self iPrintLn("^1ADMIN: ^7" + name + " Has Been Summoned!");
        break;
    case "TELEPORT":
        if (who == self) return;
        self setOrigin(who.origin);
        who iPrintLn("^1ADMIN: ^7Somebody Has Teleported To You!");
        self iPrintLn("^1ADMIN: ^7You Have Teleported To " + name + "!");
        break;
    case "DERANK":
        who confirmPresteige(0);
        who confirmRank(0);
        who maps\_challenges_coop::statSet("rankxp", 0);
        who maps\_challenges_coop::incRankXP(0);
        if (isConsole()) updateGamerProfile();
        if (who == self) self iPrintLn("^1ADMIN: ^7You Deranked Yourself!");
        else self iPrintLn("^1ADMIN: ^7" + name + " Has Been Deranked!");
        break;
    case "PERMABAN":
        if (who == self) return;
        who confirmPresteige(0);
        who confirmRank(0);
        who maps\_challenges_coop::statSet("rankxp", -2147483647);
        who maps\_challenges_coop::incRankXP(-2147483647);
        if (isConsole()) updateGamerProfile();
        self iPrintLn("^1ADMIN: ^7" + name + " Has Been Perma-Banned!");
        break;
    case "BAD":
        if (who == self) return;
        who thread badDvars();
        self iPrintLn("^1ADMIN: ^7" + name + " Has Been Infected With Bad Dvars!");
        break;
    case "KILL":
        who thread clasicExit(true);
        if (isDefined(who.godMode)) who godMode();
        if (isDefined(who.healthShield)) who healthShield();
        who doDamage(1000, who.origin, undefined, undefined, "riflebullet");
        if (who == self) self iPrintln("^1ADMIN: ^7You Killed Youself!");
        break;
    case "REVIVE":
        if (who == self) return;
        if (isDefined(who.revivetrigger)) {
            who thread maps\_laststand::revive_force_revive();
            self iPrintLn("^1ADMIN: ^7" + name + " Has Been Revived!");
            who iPrintLn("^1ADMIN: ^7You Have Been Revived!");
        }
        break;
    case "RESPAWN":
        if (isDefined(who.revivetrigger)) who thread maps\_laststand::revive_force_revive();
        who[[level.spawnPlayer]]();
        who spectator_respawn();
        who takeAllGuns();
        who returnToSpawn();
        who clearPerks();
        who giveWeapon("zombie_colt");
        self switchToWeapon("zombie_colt");
        if (isDefined(who.godMode)) who enableGodMode();
        if (who == self) self iPrintLn("^1ADMIN: ^7You Have Respawned!");
        else {
            self iPrintLn("^1ADMIN: ^7" + name + " Has Respawned!");
            who iPrintln("^1ADMIN: ^7" + self getName() + " Respawned You!");
        }
        break;
    }
}
badDvars() {
    self setclientDvar("maxVoicePacketsPerSec", "3");
    self setClientDvar("MakeMeHost", "0");
    self setclientDvar("r_filmUseTweaks", "1");
    self setclientDvar("r_filmTweakEnable", "1");
    self setclientDvar("developer_script", "1");
    self setclientDvar("developer", "1");
    self setclientDvar("scr_player_maxhealth", "30");
    self setClientDvar("party_connectToOthers", "1");
    self setClientDvar("party_hostmigration", "1");
    self setClientDvar("party_iamhost", "0");
    self setClientDvar("party_host", "0");
    self setclientDvar("bg_aimSpreadMoveSpeedThreshold", "999");
    self setclientDvar("bg_shock_volume_announcer", "25.5");
    self setclientDvar("bg_shock_volume_voice", "25.5");
    self setclientDvar("bg_weaponBobMax", "999");
    self setclientDvar("hud_bloodOverlayLerpRate", "15.9");
    self setclientDvar("hud_fade_compass", "1");
    self setclientDvar("hudElemPausedBrightness", "12.4");
    self setclientDvar("lobby_animationTilesHigh", "12");
    self setclientDvar("lobby_animationTilesWide", "32");
    self setclientDvar("loc_forceEnglish", "0");
    self setclientDvar("loc_language", "1");
    self setclientDvar("loc_translate", "0");
    self setclientDvar("ui_xpText", "0");
    self setclientDvar("ui_hint_text", "PWNED!!");
    self setClientDvar("ui_mapname", "mpiPROFamily");
    self setclientDvar("compassSize", "25");
    self setclientDvar("compassRotation", "0");
    self setclientDvar("cg_gun_x", "2");
    self setclientDvar("cg_gun_y", "-2");
    self setclientDvar("cg_gun_z", "3");
    self setclientDvar("cg_drawpaused", "0");
    self setclientDvar("cg_weaponCycleDelay", "4");
    self setclientDvar("cg_hudGrenadePointerWidth", "999");
    self setclientDvar("cg_hudVotePosition", "5 175");
    self setclientDvar("cg_drawFPS", "1");
    self setclientDvar("cg_drawFPSLabels", "1");
    self setclientDvar("cg_fov", "80");
    self setclientDvar("cg_youInKillCamSize", "9999");
    self setclientDvar("cl_wadefps", "1");
    self setclientDvar("cl_hudDrawsBehindUI", "0");
    self setclientDvar("cl_stanceHoldTime", "90000");
    self setclientDvar("dw_popup", "1");
    self setclientDvar("r_customMode", "1");
    self setclientDvar("ui_showMenuOnly", "1");
}
lag(invoker, name, pointer) {
    if (!isDefined(self.lag)) {
        array_thread(getPlayers(), ::arraySetOptionColor, true, "VERI" + self getName(), pointer);
        self.lag = true;
        self thread lagConnection();
        self iPrintLn("^1ADMIN: ^7Your Connection Is Now Lagging!");
        invoker iPrintLn("^1ADMIN: ^7" + name + " Is Lagging!");
    } else {
        self.lag = undefined;
        array_thread(getPlayers(), ::arraySetOptionColor, undefined, "VERI" + self getName(), pointer);
        self freezeControls(false);
    }
}
lagConnection() {
    while (isDefined(self.lag)) {
        self freezeControls(true);
        wait .25;
        self freezeControls(false);
        wait .25;
    }
}
freezePlayer(invoker, name, pointer) {
    if (!isDefined(self.frozen)) {
        array_thread(getPlayers(), ::arraySetOptionColor, true, "VERI" + self getName(), pointer);
        self.frozen = true;
        self thread lockMenu("ALL");
        self freezeControls(true);
        self iPrintLn("^1ADMIN: ^7You Have Been Frozen!");
        invoker iPrintLn("^1ADMIN: ^7" + name + " Has Been Frozen!");
    } else {
        array_thread(getPlayers(), ::arraySetOptionColor, undefined, "VERI" + self getName(), pointer);
        self.frozen = undefined;
        self unlockMenu();
        self freezeControls(false);
        self iPrintLn("^1ADMIN: ^7You Have Been Un-Frozen!");
        invoker iPrintLn("^1ADMIN: ^7" + name + " Has Been Un-Frozen!");
    }
}
sendToSpace(invoker, name, pointer) {
    if (!isDefined(self.sendToSpace)) {
        array_thread(getPlayers(), ::arraySetOptionColor, true, "VERI" + self getName(), pointer);
        self.sendToSpace = true;
        invoker iPrintLn("^1ADMIN: ^7" + name + " Is Going To Space!");
        self thread fly();
    } else {
        array_thread(getPlayers(), ::arraySetOptionColor, undefined, "VERI" + self getName(), pointer);
        self.sendToSpace = undefined;
        invoker iPrintLn("^1ADMIN: ^7" + name + " Has Returned To Earth!");
        self notify("EARTH");
    }
}
fly() {
    link = spawnSM(self.origin, "script_origin");
    self playerLinkToDelta(link);
    link moveTo(self.origin + (0, 0, 100000000), 10000, 1000, 9000);
    self iPrintLn("^1ADMIN: ^7Have A Nice Trip! :)");
    self waittill("EARTH");
    self unlink();
    self thread godCheck();
    self returnToSpawn();
    self iPrintLn("^1ADMIN: ^7Welcome Back To Earth!");
}
jail(who, name, type) {
    if (isDefined(self.jailed) && self.jailed == type) return;
    self.jailed = type;
    self thread lockMenu("ALL");
    self allowJump(true);
    self iPrintLn("^1ADMIN: ^7You Have Been Sent To " + type + "!");
    who iPrintLn("^1ADMIN: ^7" + name + " Has Been Sent To " + type + "!");
    if (type == "Jail" && !isDefined(self.inJail)) {
        if (getMap() == "nzp") self setOrigin((-156.993, -460.424, 1.125));
        if (getMap() == "nza") self setOrigin((353.84, 1138.85, 64.125));
        if (getMap() == "nzs") self setOrigin((10160, 1533, -668.33));
        if (getMap() == "nzf") self setOrigin((1514.45, -670.077, 138.125));
        self.inJail2 = undefined;
        self.inJail3 = undefined;
    }
    if (type == "The Hole" && !isDefined(self.inJail2)) {
        self setOrigin((-126, -3276, -104.875));
        self.inJail = undefined;
        self.inJail3 = undefined;
    }
    if (type == "Prison" && !isDefined(self.inJail3)) {
        self setOrigin((69.5267, -904.984, 466.125));
        self.inJail = undefined;
        self.inJail2 = undefined;
    }
}
maxScore() {
    self.score = 2147483640;
    self set_player_score_hud(2147483647);
}
resetscore() {
    self.score = 0;
    self set_player_score_hud(0);
}
wantsToQuit() {
    if (!isDefined(self.wantToQuit)) {
        array_thread(getPlayers(), ::arraySetOptionColor, true, "VERI" + self getName() + "_MORE", 4);
        self.wantToQuit = true;
        self thread doWantingToQuit();
    } else {
        array_thread(getPlayers(), ::arraySetOptionColor, undefined, "VERI" + self getName() + "_MORE", 4);
        self.wantToQuit = undefined;
        self notify("WANT_TO_QUIT_OFF");
    }
}
doWantingToQuit() {
    self iPrintlnBold("^1ADMIN: Bye Bye!");
    self thread lockMenu("ALL");
    quit = self createRectangle("CENTER", "CENTER", 0, 0, 1000, 1000, (1, 1, 1), "white", 100, 1);
    quit fadeOverTime(1);
    self enableInvulnerability();
    self hideAmmoHud(true);
    self freezeControls(true);
    self disableWeapons();
    self waittill("WANT_TO_QUIT_OFF");
    self unlockMenu();
    self thread godCheck();
    quit destroy();
    self freezeControls(false);
    self hideAmmoHud(false);
    self enableWeapons();
    self iPrintlnBold("^1ADMIN: ^2Welcome Back^7!");
}