#include common_scripts\utility;
#include maps\_utility;
#include maps\_hud_util;

//ZOMBIE MODE FUNCTIONS\\
randyWobble() {
    level endon("SKYBASE_FAIL");
    while (isDefined(self)) {
        tm = randomFloatRange(2.5, 5);
        yaw = randomInt(360);
        if (yaw > 300) yaw = 300;
        else if (yaw < 60) yaw = 60;
        yaw = self.angles[1] + yaw;
        self rotateTo((-60 + randomInt(120), yaw, -45 + randomInt(90)), tm, tm * .5, tm * .5);
        wait(randomFloat(tm - .1));
    }
}
powerup_drop(drop_point) {
    rand_drop = randomint(100);
    if (level.powerup_drop_count >= level.zombie_vars["zombie_powerup_drop_max_per_round"]) {
        println("^3POWERUP DROP EXCEEDED THE MAX PER ROUND!");
        return;
    }
    if (!isDefined(level.zombie_include_powerups) || level.zombie_include_powerups.size == 0) return;
    if (rand_drop > 2) {
        if (!level.zombie_vars["zombie_drop_item"]) return;
        debug = "score";
    } else debug = "random";
    playable_area = getentarray("playable_area", "targetname");
    powerup = network_safe_spawn("powerup", 1, "script_model", drop_point + (0, 0, 40));
    valid_drop = false;
    for (i = 0; i < playable_area.size; i++)
        if (powerup istouching(playable_area[i])) valid_drop = true;
    if (!valid_drop) {
        powerup delete();
        return;
    }
    powerup powerup_setup();
    level.powerup_drop_count++;
    powerup thread powerup_timeout();
    powerup thread powerup_wobble();
    powerup thread powerup_grab();
    level.zombie_vars["zombie_drop_item"] = 0;
}
network_safe_spawn(id, max, classname, origin) {
    network_safe_init(id, max);
    return (network_choke_action(id, ::_network_safe_spawn, classname, origin));
}
network_safe_init(id, max) {
    if (!IsDefined(level.zombie_network_choke_ids_max) || !isDefined(level.zombie_network_choke_ids_max[id])) network_choke_init(id, max);
    assert(max == level.zombie_network_choke_ids_max[id]);
}
network_choke_init(id, max) {
    if (!IsDefined(level.zombie_network_choke_ids_max)) {
        level.zombie_network_choke_ids_max = [];
        level.zombie_network_choke_ids_count = [];
    }
    level.zombie_network_choke_ids_max[id] = max;
    level.zombie_network_choke_ids_count[id] = 0;
    level thread network_choke_thread(id);
}
network_choke_thread(id) {
    for (;;) {
        wait_network_frame();
        wait_network_frame();
        level.zombie_network_choke_ids_count[id] = 0;
    }
}
powerup_setup() {
    powerup = get_next_powerup();
    struct = level.zombie_powerups[powerup];
    self SetModel(struct.model_name);
    playsoundatposition("spawn_powerup", self.origin);
    self.powerup_name = struct.powerup_name;
    self.hint = struct.hint;
    if (IsDefined(struct.fx)) self.fx = struct.fx;
    self PlayLoopSound("spawn_powerup_loop");
}
get_next_powerup() {
    if (level.zombie_powerup_index >= level.zombie_powerup_array.size) {
        level.zombie_powerup_index = 0;
        randomize_powerups();
    }
    powerup = level.zombie_powerup_array[level.zombie_powerup_index];
    while (powerup == "carpenter" && get_num_window_destroyed() < 5) {
        if (level.zombie_powerup_index >= level.zombie_powerup_array.size) {
            level.zombie_powerup_index = 0;
            randomize_powerups();
        }
        powerup = level.zombie_powerup_array[level.zombie_powerup_index];
        level.zombie_powerup_index++;
        if (powerup != "carpenter") return powerup;
        wait(0.05);
    }
    level.zombie_powerup_index++;
    return powerup;
}
randomize_powerups() {
    level.zombie_powerup_array = array_randomize(level.zombie_powerup_array);
}
get_num_window_destroyed() {
    num = 0;
    for (i = 0; i < level.exterior_goals.size; i++)
        if (all_chunks_destroyed(level.exterior_goals[i].barrier_chunks)) num += 1;
    return num;
}
all_chunks_destroyed(barrier_chunks) {
    for (i = 0; i < barrier_chunks.size; i++) {
        if (!barrier_chunks[i].destroyed) return false;
        if (IsDefined(barrier_chunks[i].target_by_zombie)) return false;
    }
    return true;
}
powerup_timeout() {
    self endon("powerup_grabbed");
    wait 15;
    for (i = 0; i < 40; i++) {
        if (i % 2) self hide();
        else self show();
        if (i < 15) wait 0.5;
        else if (i < 25) wait 0.25;
        else wait 0.1;
    }
    self notify("powerup_timedout");
    self delete();
}
powerup_wobble(arg) {
    self endon("powerup_grabbed");
    self endon("powerup_timedout");
    if (isdefined(self) && !isDefined(arg)) playfxontag(level._effect["powerup_on"], self, "tag_origin");
    while (isdefined(self)) {
        waittime = randomfloatrange(2.5, 5);
        yaw = RandomInt(360);
        if (yaw > 300) yaw = 300;
        else if (yaw < 60) yaw = 60;
        yaw = self.angles[1] + yaw;
        self rotateto((-60 + randomint(120), yaw, -45 + randomint(90)), waittime, waittime * 0.5, waittime * 0.5);
        wait randomfloat(waittime - 0.1);
    }
}
powerup_grab() {
    self endon("powerup_timedout");
    self endon("powerup_grabbed");
    while (isdefined(self)) {
        players = getPlayers();
        for (i = 0; i < players.size; i++) {
            if (distance(players[i].origin, self.origin) < 64 && !players[i].is_zombie) {
                playfx(level._effect["powerup_grabbed"], self.origin);
                playfx(level._effect["powerup_grabbed_wave"], self.origin);
                if (IsDefined(level.zombie_powerup_grab_func)) level thread[[level.zombie_powerup_grab_func]]();
                else {
                    switch (self.powerup_name) {
                    case "nuke":
                        level thread nuke_powerup(self);
                        players[i] thread powerup_vo("nuke");
                        zombies = getaiarray("axis");
                        players[i].zombie_nuked = get_array_of_closest(self.origin, zombies);
                        players[i] notify("nuke_triggered");
                        break;
                    case "full_ammo":
                        level thread full_ammo_powerup(self);
                        players[i] thread powerup_vo("full_ammo");
                        break;
                    case "double_points":
                        level thread double_points_powerup(self);
                        players[i] thread powerup_vo("double_points");
                        break;
                    case "insta_kill":
                        level thread insta_kill_powerup(self);
                        players[i] thread powerup_vo("insta_kill");
                        break;
                    case "carpenter":
                        level thread start_carpenter();
                        players[i] thread powerup_vo("carpenter");
                        break;
                    default:
                        println("Unrecognized poweup.");
                        break;
                    }
                }
                wait(0.1);
                playsoundatposition("powerup_grabbed", self.origin);
                self stoploopsound();
                self delete();
                self notify("powerup_grabbed");
            }
        }
        wait 0.1;
    }
}
nuke_powerup(drop_item) {
    zombies = getaispeciesarray("axis");
    PlayFx(drop_item.fx, drop_item.origin);
    level thread nuke_flash();
    zombies = get_array_of_closest(drop_item.origin, zombies);
    for (i = 0; i < zombies.size; i++) {
        wait(randomfloatrange(0.1, 0.7));
        if (!IsDefined(zombies[i])) continue;
        if (zombies[i].animname == "boss_zombie") continue;
        if (is_magic_bullet_shield_enabled(zombies[i])) continue;
        if (i < 5 && !(zombies[i] enemy_is_dog())) zombies[i] thread animscripts\death::flame_death_fx();
        if (!(zombies[i] enemy_is_dog())) zombies[i] zombie_head_gib();
        zombies[i] dodamage(zombies[i].health + 666, zombies[i].origin);
        playsoundatposition("nuked", zombies[i].origin);
    }
    players = getPlayers();
    for (i = 0; i < players.size; i++) {
        players[i].score += 400;
        players[i].score_total += 400;
        players[i] set_player_score_hud();
    }
}
nuke_flash() {
    players = getPlayers();
    for (i = 0; i < players.size; i++) players[i] play_sound_2d("nuke_flash");
    level thread devil_dialog_delay();
    fadetowhite = newhudelem();
    fadetowhite.x = 0;
    fadetowhite.y = 0;
    fadetowhite.alpha = 0;
    fadetowhite.horzAlign = "fullscreen";
    fadetowhite.vertAlign = "fullscreen";
    fadetowhite.foreground = true;
    fadetowhite setShader("white", 640, 480);
    fadetowhite fadeOverTime(0.2);
    fadetowhite.alpha = 0.8;
    wait 0.5;
    fadetowhite FadeOverTime(1.0);
    fadetowhite.alpha = 0;
    wait 1.1;
    fadetowhite destroy();
}
devil_dialog_delay() {
    wait(1.8);
    level thread play_devil_dialog("nuke_vox");
}
play_devil_dialog(sound_to_play) {
    if (!IsDefined(level.devil_is_speaking)) level.devil_is_speaking = 0;
    if (level.devil_is_speaking == 0) {
        level.devil_is_speaking = 1;
        play_sound_2D(sound_to_play);
        wait 2.0;
        level.devil_is_speaking = 0;
    }
}
play_sound_2D(sound) {
    level thread really_play_2D_sound(sound);
}
really_play_2D_sound(sound) {
    temp_ent = spawn("script_origin", (0, 0, 0));
    temp_ent playsound(sound, sound + "wait");
    temp_ent waittill(sound + "wait");
    wait(0.05);
    temp_ent delete();
}
is_magic_bullet_shield_enabled(ent) {
    if (!IsDefined(ent)) return false;
    return (IsDefined(ent.magic_bullet_shield) && ent.magic_bullet_shield == true);
}
enemy_is_dog() {
    return (self.type == "dog");
}
powerup_vo(type) {
    self endon("death");
    self endon("disconnect");
    index = get_player_index(self);
    sound = undefined;
    rand = randomintrange(0, 3);
    vox_rand = randomintrange(1, 100);
    percentage = 1;
    if (!isdefined(level.player_is_speaking)) level.player_is_speaking = 0;
    wait(randomfloatrange(1, 2));
    switch (type) {
    case "nuke":
        if (vox_rand <= percentage) {} else sound = "plr_" + index + "_vox_powerup_nuke_" + rand;
        break;
    case "insta_kill":
        if (vox_rand <= percentage) {} else sound = "plr_" + index + "_vox_powerup_insta_" + rand;
        break;
    case "full_ammo":
        if (vox_rand <= percentage) {} else {
            sound = "plr_" + index + "_vox_powerup_ammo_" + rand;
        }
        break;
    case "double_points":
        if (vox_rand <= percentage) {} else sound = "plr_" + index + "_vox_powerup_double_" + rand;
        break;
    case "carpenter":
        if (vox_rand <= percentage) {} else sound = "plr_" + index + "_vox_powerup_carp_" + rand;
        break;
    }
    if (level.player_is_speaking != 1 && isDefined(sound)) {
        level.player_is_speaking = 1;
        self playsound(sound, "sound_done");
        self waittill("sound_done");
        level.player_is_speaking = 0;
    }
}
full_ammo_powerup(drop_item) {
    players = getPlayers();
    for (i = 0; i < players.size; i++) {
        primaryWeapons = players[i] GetWeaponsList();
        for (x = 0; x < primaryWeapons.size; x++) players[i] GiveMaxAmmo(primaryWeapons[x]);
    }
    level thread full_ammo_on_hud(drop_item);
}
full_ammo_on_hud(drop_item) {
    self endon("disconnect");
    hudelem = createFontString("objective", 2);
    hudelem setPoint("TOP", undefined, 0, level.zombie_vars["zombie_timer_offset"] - (level.zombie_vars["zombie_timer_offset_interval"] * 2));
    hudelem.sort = 0.5;
    hudelem.alpha = 0;
    hudelem fadeovertime(0.5);
    hudelem.alpha = 1;
    hudelem.label = "Max Ammo!";
    hudelem thread full_ammo_move_hud();
}
full_ammo_move_hud() {
    players = getPlayers();
    level thread play_devil_dialog("ma_vox");
    for (i = 0; i < players.size; i++) players[i] playsound("full_ammo");
    wait 0.5;
    move_fade_time = 1.5;
    self FadeOverTime(move_fade_time);
    self MoveOverTime(move_fade_time);
    self.y = 270;
    self.alpha = 0;
    wait move_fade_time;
    self destroy();
}
double_points_powerup(drop_item) {
    level notify("powerup points scaled");
    level endon("powerup points scaled");
    level thread point_doubler_on_hud(drop_item);
    level.zombie_vars["zombie_point_scalar"] = 2;
    wait 30;
    level.zombie_vars["zombie_point_scalar"] = 1;
}
point_doubler_on_hud(drop_item) {
    self endon("disconnect");
    if (level.zombie_vars["zombie_powerup_point_doubler_on"]) {
        level.zombie_vars["zombie_powerup_point_doubler_time"] = 30;
        level thread play_devil_dialog("dp_vox");
        return;
    }
    level.zombie_vars["zombie_powerup_point_doubler_on"] = true;
    level thread time_remaining_on_point_doubler_powerup();
}
time_remaining_on_point_doubler_powerup() {
    temp_ent = spawn("script_origin", (0, 0, 0));
    temp_ent playloopsound("double_point_loop");
    level thread play_devil_dialog("dp_vox");
    while (level.zombie_vars["zombie_powerup_point_doubler_time"] >= 0) {
        wait 0.1;
        level.zombie_vars["zombie_powerup_point_doubler_time"] = level.zombie_vars["zombie_powerup_point_doubler_time"] - 0.1;
    }
    level.zombie_vars["zombie_powerup_point_doubler_on"] = false;
    players = getPlayers();
    for (i = 0; i < players.size; i++) players[i] playsound("points_loop_off");
    temp_ent stoploopsound(2);
    level.zombie_vars["zombie_powerup_point_doubler_time"] = 30;
    temp_ent delete();
}
insta_kill_powerup(drop_item) {
    level notify("powerup instakill");
    level endon("powerup instakill");
    level thread insta_kill_on_hud(drop_item);
    level.zombie_vars["zombie_insta_kill"] = 1;
    wait 30;
    level.zombie_vars["zombie_insta_kill"] = 0;
    players = getPlayers();
    for (i = 0; i < players.size; i++) players[i] notify("insta_kill_over");
}
insta_kill_on_hud(drop_item) {
    self endon("disconnect");
    if (level.zombie_vars["zombie_powerup_insta_kill_on"]) {
        level.zombie_vars["zombie_powerup_insta_kill_time"] = 30;
        level thread play_devil_dialog("insta_vox");
        return;
    }
    level.zombie_vars["zombie_powerup_insta_kill_on"] = true;
    level thread time_remaning_on_insta_kill_powerup();
}
time_remaning_on_insta_kill_powerup() {
    level thread play_devil_dialog("insta_vox");
    temp_enta = spawn("script_origin", (0, 0, 0));
    temp_enta playloopsound("insta_kill_loop");
    while (level.zombie_vars["zombie_powerup_insta_kill_time"] >= 0) {
        wait 0.1;
        level.zombie_vars["zombie_powerup_insta_kill_time"] = level.zombie_vars["zombie_powerup_insta_kill_time"] - 0.1;
    }
    players = getPlayers();
    for (i = 0; i < players.size; i++) players[i] playsound("insta_kill");
    temp_enta stoploopsound(2);
    level.zombie_vars["zombie_powerup_insta_kill_on"] = false;
    level.zombie_vars["zombie_powerup_insta_kill_time"] = 30;
    temp_enta delete();
}
start_carpenter() {
    level thread play_devil_dialog("carp_vox");
    window_boards = getstructarray("exterior_goal", "targetname");
    total = level.exterior_goals.size;
    carp_ent = spawn("script_origin", (0, 0, 0));
    carp_ent playloopsound("carp_loop");
    while (true) {
        windows = get_closest_window_repair(window_boards, self.origin);
        if (!IsDefined(windows)) {
            carp_ent stoploopsound(1);
            carp_ent playsound("carp_end", "sound_done");
            carp_ent waittill("sound_done");
            break;
        } else window_boards = array_remove(window_boards, windows);
        while (1) {
            if (all_chunks_intact(windows.barrier_chunks)) {
                break;
            }
            chunk = get_random_destroyed_chunk(windows.barrier_chunks);
            if (!IsDefined(chunk)) break;
            windows thread replace_chunk(chunk, false, true);
            windows.clip enable_trigger();
            windows.clip DisconnectPaths();
            wait_network_frame();
            wait(0.05);
        }
        wait_network_frame();
    }
    players = get_players();
    for (i = 0; i < players.size; i++) {
        players[i].score += 200;
        players[i].score_total += 200;
        players[i] set_player_score_hud();
    }
    carp_ent delete();
}
get_closest_window_repair(windows, origin) {
    current_window = undefined;
    shortest_distance = undefined;
    for (i = 0; i < windows.size; i++) {
        if (all_chunks_intact(windows[i].barrier_chunks)) continue;
        if (!IsDefined(current_window)) {
            current_window = windows[i];
            shortest_distance = DistanceSquared(current_window.origin, origin);
        } else {
            if (DistanceSquared(windows[i].origin, origin) < shortest_distance) {
                current_window = windows[i];
                shortest_distance = DistanceSquared(windows[i].origin, origin);
            }
        }
    }
    return current_window;
}
all_chunks_intact(barrier_chunks) {
    for (i = 0; i < barrier_chunks.size; i++)
        if (barrier_chunks[i].destroyed && !IsDefined(barrier_chunks[i].mid_repair)) return false;
    return true;
}
get_random_destroyed_chunk(barrier_chunks) {
    chunk = undefined;
    chunks = get_destroyed_chunks(barrier_chunks);
    if (IsDefined(chunks)) return chunks[RandomInt(chunks.size)];
    return undefined;
}
get_destroyed_chunks(barrier_chunks) {
    array = [];
    for (i = 0; i < barrier_chunks.size; i++)
        if (barrier_chunks[i].destroyed && !isdefined(barrier_chunks[i].mid_repair)) array[array.size] = barrier_chunks[i];
    if (array.size == 0) return undefined;
    return array;
}
replace_chunk(chunk, has_perk, via_powerup) {
    if (!IsDefined(has_perk)) has_perk = false;
    assert(IsDefined(chunk.og_origin));
    assert(!IsDefined(chunk.mid_repair));
    chunk.mid_repair = true;
    chunk Show();
    sound = "rebuild_barrier_hover";
    if (IsDefined(chunk.script_presound)) sound = chunk.script_presound;
    if (!isdefined(via_powerup)) play_sound_at_pos(sound, chunk.origin);
    only_z = (chunk.origin[0], chunk.origin[1], chunk.og_origin[2]);
    if (has_perk == true) {
        chunk RotateTo((0, 0, 0), 0.15);
        chunk waittill_notify_or_timeout("rotatedone", 1);
        wait(0.1);
    } else {
        chunk MoveTo(only_z, 0.15);
        chunk RotateTo((0, 0, 0), 0.3);
        chunk waittill_notify_or_timeout("rotatedone", 1);
        wait(0.2);
    }
    if (has_perk == true) chunk MoveTo(chunk.og_origin, 0.05);
    else chunk MoveTo(chunk.og_origin, 0.1);
    chunk waittill_notify_or_timeout("movedone", 1);
    assert(chunk.origin == chunk.og_origin);
    chunk.target_by_zombie = undefined;
    chunk.destroyed = false;
    assert(chunk.mid_repair == true);
    chunk.mid_repair = undefined;
    sound = "barrier_rebuild_slam";
    if (IsDefined(self.script_ender)) sound = self.script_ender;
    chunk Solid();
    fx = "wood_chunk_destory";
    if (IsDefined(self.script_fxid)) fx = self.script_fxid;
    if (!IsDefined(via_powerup)) {
        play_sound_at_pos(sound, chunk.origin);
        playfx(level._effect[fx], chunk.origin);
    }
    if (!Isdefined(self.clip)) chunk Disconnectpaths();
}
enable_trigger(arg) {
    if (!isDefined(self.disabled) || !self.disabled) {
        if (isDefined(arg)) {
            self.disabled = true;
            self.origin = self.origin - (0, 0, 10000);
        }
        return;
    }
    self.disabled = false;
    self.origin = self.origin + (0, 0, 10000);
}
play_sound_at_pos(ref, pos, ent) {
    if (IsDefined(ent)) {
        if (IsDefined(ent.script_soundalias)) {
            PlaySoundAtPosition(ent.script_soundalias, pos);
            return;
        }
        if (IsDefined(self.script_sound)) ref = self.script_sound;
    }
    if (ref == "none") return;
    if (!IsDefined(level.zombie_sounds[ref])) {
        AssertMsg("Sound \"" + ref + "\" was not put to the zombie sounds list,please use add_sound(ref,alias)at the start of your level.");
        return;
    }
    PlaySoundAtPosition(level.zombie_sounds[ref], pos);
}
zombie_damage(mod, hit_location, hit_origin, player) {
    if (is_magic_bullet_shield_enabled(self)) return;
    player.use_weapon_type = mod;
    if (isDefined(self.marked_for_death)) return;
    if (!IsDefined(player)) return;
    if (self zombie_flame_damage(mod, player))
        if (self zombie_give_flame_damage_points()) player player_add_points("damage", mod, hit_location, self enemy_is_dog());
        else if (self is_tesla_damage(mod)) {
        self tesla_damage_init(hit_location, hit_origin, player);
        return;
    } else player player_add_points("damage", mod, hit_location, self enemy_is_dog());
    if (mod == "MOD_GRENADE" || mod == "MOD_GRENADE_SPLASH") {
        if (isdefined(player) && isalive(player)) self DoDamage(level.round_number + randomintrange(100, 500), self.origin, player);
        else self DoDamage(level.round_number + randomintrange(100, 500), self.origin, undefined);
    } else if (mod == "MOD_PROJECTILE" || mod == "MOD_EXPLOSIVE" || mod == "MOD_PROJECTILE_SPLASH" || mod == "MOD_PROJECTILE_SPLASH") {
        if (isdefined(player) && isalive(player)) self DoDamage(level.round_number * randomintrange(0, 100), self.origin, player);
        else self DoDamage(level.round_number * randomintrange(0, 100), self.origin, undefined);
    } else if (mod == "MOD_ZOMBIE_BETTY") {
        if (isdefined(player) && isalive(player)) self DoDamage(level.round_number * randomintrange(100, 200), self.origin, player);
        else self DoDamage(level.round_number * randomintrange(100, 200), self.origin, undefined);
    }
    if (IsDefined(self.a.gib_ref) && (self.a.gib_ref == "no_legs") && isalive(self)) {
        if (isdefined(player)) {
            rand = randomintrange(0, 100);
            if (rand < 10) {
                index = get_player_index(player);
                plr = "plr_" + index + "_";
                player thread create_and_play_dialog(plr, "vox_crawl_spawn", 0.25, "resp_cspawn");
            }
        }
    } else if (IsDefined(self.a.gib_ref) && ((self.a.gib_ref == "right_arm") || (self.a.gib_ref == "left_arm"))) {
        if (self.has_legs && isalive(self)) {
            if (isdefined(player)) {
                rand = randomintrange(0, 100);
                if (rand < 3) {
                    index = get_player_index(player);
                    plr = "plr_" + index + "_";
                    player thread create_and_play_dialog(plr, "vox_shoot_limb", 0.25);
                }
            }
        }
    }
    self thread check_for_instakill(player);
}
zombie_flame_damage(mod, player) {
    if (mod == "MOD_BURNED") {
        self.moveplaybackrate = 0.8;
        if (!IsDefined(self.is_on_fire) || (Isdefined(self.is_on_fire) && !self.is_on_fire)) self thread damage_on_fire(player);
        do_flame_death = true;
        dist = 100 * 100;
        ai = GetAiArray("axis");
        for (i = 0; i < ai.size; i++) {
            if (IsDefined(ai[i].is_on_fire) && ai[i].is_on_fire) {
                if (DistanceSquared(ai[i].origin, self.origin) < dist) {
                    do_flame_death = false;
                    break;
                }
            }
        }
        if (do_flame_death) self thread animscripts\death::flame_death_fx();
        return true;
    }
    return false;
}
zombie_give_flame_damage_points() {
    if (GetTime() > self.flame_damage_time) {
        self.flame_damage_time = GetTime() + level.zombie_vars["zombie_flame_dmg_point_delay"];
        return true;
    }
    return false;
}
damage_on_fire(player) {
    self endon("death");
    self endon("stop_flame_damage");
    wait(2);
    while (isdefined(self.is_on_fire) && self.is_on_fire) {
        if (level.round_number < 6) dmg = level.zombie_health * RandomFloatRange(0.2, 0.3);
        else if (level.round_number < 9) dmg = level.zombie_health * RandomFloatRange(0.15, 0.25);
        else if (level.round_number < 11) dmg = level.zombie_health * RandomFloatRange(0.1, 0.2);
        else dmg = level.zombie_health * RandomFloatRange(0.1, 0.15);
        if (Isdefined(player) && Isalive(player)) self DoDamage(dmg, self.origin, player);
        else self DoDamage(dmg, self.origin, level);
        wait(randomfloatrange(1.0, 3.0));
    }
}
player_add_points(event, mod, hit_location, is_dog) {
    if (level.intermission) return;
    if (!is_player_valid(self)) return;
    points = 0;
    switch (event) {
    case "death":
        points = level.zombie_vars["zombie_score_kill"];
        points += player_add_points_kill_bonus(mod, hit_location);
        if (IsDefined(self.kill_tracker)) self.kill_tracker++;
        else self.kill_tracker = 1;
        self.stats["kills"] = self.kill_tracker;
        if (mod == "MOD_MELEE" && self hasperk("specialty_altmelee")) self achievement_notify("DLC3_ZOMBIE_BOWIE_KILLS");
        if (level.zombie_vars["zombie_powerup_insta_kill_on"] == 1 && mod == "MOD_UNKNOWN") points = points * 2;
        break;
    case "damage":
        points = level.zombie_vars["zombie_score_damage"];
        break;
    case "damage_ads":
        points = Int(level.zombie_vars["zombie_score_damage"] * 1.25);
        break;
    default:
        assertex(0, "Unknown point event");
        break;
    }
    points = round_up_to_ten(points) * level.zombie_vars["zombie_point_scalar"];
    self.score += points;
    self.score_total += points;
    self.stats["score"] = self.score_total;
    self set_player_score_hud();
}
player_add_points_kill_bonus(mod, hit_location) {
    if (mod == "MOD_MELEE") return level.zombie_vars["zombie_score_bonus_melee"];
    if (mod == "MOD_BURNED") return level.zombie_vars["zombie_score_bonus_burn"];
    score = 0;
    switch (hit_location) {
    case "head":
    case "helmet":
        score = level.zombie_vars["zombie_score_bonus_head"];
        break;
    case "neck":
        score = level.zombie_vars["zombie_score_bonus_neck"];
        break;
    case "torso_upper":
    case "torso_lower":
        score = level.zombie_vars["zombie_score_bonus_torso"];
        break;
    }
    return score;
}
achievement_notify(notify_name) {
    self notify(notify_name);
}
is_tesla_damage(mod) {
    return ((IsDefined(self.damageweapon) && (self.damageweapon == "tesla_gun" || self.damageweapon == "tesla_gun_upgraded")) && (mod == "MOD_PROJECTILE" || mod == "MOD_PROJECTILE_SPLASH"));
}
tesla_damage_init(hit_location, hit_origin, player) {
    player endon("disconnect");
    if (IsDefined(player.tesla_enemies_hit) && player.tesla_enemies_hit > 0) return;
    if (IsDefined(self.zombie_tesla_hit) && self.zombie_tesla_hit) return;
    player.tesla_enemies = undefined;
    player.tesla_enemies_hit = 1;
    player.tesla_powerup_dropped = false;
    player.tesla_arc_count = 0;
    self tesla_arc_damage(self, player, 1);
    if (player.tesla_enemies_hit >= 4) player thread tesla_killstreak_sound();
    player.tesla_enemies_hit = 0;
}
tesla_arc_damage(source_enemy, player, arc_num) {
    player endon("disconnect");
    tesla_flag_hit(self, true);
    wait_network_frame();
    self thread tesla_do_damage(source_enemy, arc_num, player);
    radius_decay = level.zombie_vars["tesla_radius_decay"] * arc_num;
    enemies = tesla_get_enemies_in_area(self GetTagOrigin("j_head"), level.zombie_vars["tesla_radius_start"] - radius_decay, player);
    tesla_flag_hit(enemies, true);
    for (i = 0; i < enemies.size; i++) {
        if (enemies[i] == self) continue;
        if (tesla_end_arc_damage(arc_num + 1, player.tesla_enemies_hit)) {
            tesla_flag_hit(enemies[i], false);
            continue;
        }
        player.tesla_enemies_hit++;
        enemies[i] tesla_arc_damage(self, player, arc_num + 1);
    }
}
tesla_end_arc_damage(arc_num, enemies_hit_num) {
    if (arc_num >= level.zombie_vars["tesla_max_arcs"]) return true;
    if (enemies_hit_num >= level.zombie_vars["tesla_max_enemies_killed"]) return true;
    radius_decay = level.zombie_vars["tesla_radius_decay"] * arc_num;
    if (level.zombie_vars["tesla_radius_start"] - radius_decay <= 0) return true;
    return false;
}
tesla_flag_hit(enemy, hit) {
    if (IsArray(enemy))
        for (i = 0; i < enemy.size; i++) enemy[i].zombie_tesla_hit = hit;
    else enemy.zombie_tesla_hit = hit;
}
tesla_do_damage(source_enemy, arc_num, player) {
    player endon("disconnect");
    if (arc_num > 1) wait(RandomFloat(0.2, 0.6) * arc_num);
    if (!isDefined(self) || !isAlive(self)) return;
    if (!self enemy_is_dog()) {
        if (self.has_legs) self.deathanim = random(level._zombie_tesla_death[self.animname]);
        else self.deathanim = random(level._zombie_tesla_crawl_death[self.animname]);
    } else self.a.nodeath = undefined;
    if (source_enemy != self) {
        if (player.tesla_arc_count > 3) {
            wait_network_frame();
            player.tesla_arc_count = 0;
        }
        player.tesla_arc_count++;
        source_enemy tesla_play_arc_fx(self);
    }
    while (player.tesla_network_death_choke > level.zombie_vars["tesla_network_death_choke"]) wait(0.05);
    if (!IsDefined(self) || !IsAlive(self)) return;
    player.tesla_network_death_choke++;
    self.tesla_death = true;
    self tesla_play_death_fx(arc_num);
    origin = source_enemy.origin;
    if (source_enemy == self || !IsDefined(origin)) origin = player.origin;
    if (!IsDefined(self) || !IsAlive(self)) return;
    if (self.animname == "boss_zombie") {
        if (!isDefined(level.boss_tesla_damage)) level.boss_tesla_damage = 1000;
        self DoDamage(level.boss_tesla_damage, origin, player);
    } else self DoDamage(self.health + 666, origin, player);
    player player_add_points("death", "", "");
    if (!player.tesla_powerup_dropped && player.tesla_enemies_hit >= level.zombie_vars["tesla_kills_for_powerup"]) {
        player.tesla_powerup_dropped = true;
        level.zombie_vars["zombie_drop_item"] = 1;
        level thread powerup_drop(self.origin);
    }
}
tesla_get_enemies_in_area(origin, distance, player) {
    distance_squared = distance * distance;
    enemies = [];
    if (!IsDefined(player.tesla_enemies)) {
        player.tesla_enemies = GetAiSpeciesArray("axis", "all");
        player.tesla_enemies = get_array_of_closest(origin, player.tesla_enemies);
    }
    zombies = player.tesla_enemies;
    if (IsDefined(zombies)) {
        for (i = 0; i < zombies.size; i++) {
            if (!IsDefined(zombies[i])) continue;
            test_origin = zombies[i] GetTagOrigin("j_head");
            if (IsDefined(zombies[i].zombie_tesla_hit) && zombies[i].zombie_tesla_hit == true) continue;
            if (is_magic_bullet_shield_enabled(zombies[i])) continue;
            if (DistanceSquared(origin, test_origin) > distance_squared) continue;
            if (!BulletTracePassed(origin, test_origin, false, undefined)) continue;
            enemies[enemies.size] = zombies[i];
        }
    }
    return enemies;
}
tesla_killstreak_sound() {
    self endon("disconnect");
    index = get_player_index(self);
    plr = "plr_" + index + "_";
    self thread play_tesla_dialog(plr);
    wait(3.5);
    level clientNotify("TGH");
}
tesla_play_arc_fx(target) {
    if (!IsDefined(self) || !IsDefined(target)) {
        wait(level.zombie_vars["tesla_arc_travel_time"]);
        return;
    }
    tag = "J_SpineUpper";
    if (self enemy_is_dog()) tag = "J_Spine1";
    target_tag = "J_SpineUpper";
    if (target enemy_is_dog()) target_tag = "J_Spine1";
    origin = self GetTagOrigin(tag);
    target_origin = target GetTagOrigin(target_tag);
    distance_squared = level.zombie_vars["tesla_min_fx_distance"] * level.zombie_vars["tesla_min_fx_distance"];
    if (DistanceSquared(origin, target_origin) < distance_squared) return;
    fxOrg = Spawn("script_model", origin);
    fxOrg SetModel("tag_origin");
    fx = PlayFxOnTag(level._effect["tesla_bolt"], fxOrg, "tag_origin");
    playsoundatposition("tesla_bounce", fxOrg.origin);
    fxOrg MoveTo(target_origin, level.zombie_vars["tesla_arc_travel_time"]);
    fxOrg waittill("movedone");
    fxOrg delete();
}
network_safe_play_fx_on_tag(id, max, fx, entity, tag) {
    network_safe_init(id, max);
    network_choke_action(id, ::_network_safe_play_fx_on_tag, fx, entity, tag);
}
_network_safe_play_fx_on_tag(fx, entity, tag) {
    if (network_entity_valid(entity)) PlayFxOnTag(fx, entity, tag);
}
network_entity_valid(entity) {
    if (!IsDefined(entity)) return false;
    return true;
}
check_for_instakill(player) {
    if (IsDefined(player) && IsAlive(player) && level.zombie_vars["zombie_insta_kill"]) {
        if (is_magic_bullet_shield_enabled(self)) return;
        if (self.animname == "boss_zombie") return;
        if (player.use_weapon_type == "MOD_MELEE") player.last_kill_method = "MOD_MELEE";
        else player.last_kill_method = "MOD_UNKNOWN";
        if (flag("dog_round")) {
            self DoDamage(self.health + 666, self.origin, player);
            player notify("zombie_killed");
        } else {
            self zombie_head_gib();
            self DoDamage(self.health + 666, self.origin, player);
            player notify("zombie_killed");
        }
    }
}
play_tesla_dialog(player_index) {
    waittime = 0.25;
    if (!IsDefined(self.vox_kill_tesla)) {
        num_variants = get_number_variants(player_index + "vox_kill_tesla");
        self.vox_kill_tesla = [];
        for (i = 0; i < num_variants; i++) self.vox_kill_tesla[self.vox_kill_tesla.size] = "vox_kill_tesla_" + i;
        self.vox_kill_tesla_available = self.vox_kill_tesla;
    }
    if (!isdefined(level.player_is_speaking)) level.player_is_speaking = 0;
    sound_to_play = random(self.vox_kill_tesla_available);
    self do_player_playdialog(player_index, sound_to_play, waittime);
    self.vox_kill_tesla_available = array_delete(self.vox_kill_tesla_available, sound_to_play);
    if (self.vox_kill_tesla_available.size < 1) self.vox_kill_tesla_available = self.vox_kill_tesla;
}
network_choke_action(id, choke_action, arg1, arg2, arg3) {
    AssertEx(IsDefined(level.zombie_network_choke_ids_max[id]), "Network Choke: " + id + " undefined");
    while (!network_choke_safe(id)) wait(0.05);
    level.zombie_network_choke_ids_count[id]++;
    if (!isDefined(arg1)) return ([
        [choke_action]
    ]());
    if (!isDefined(arg2)) return ([
        [choke_action]
    ](arg1));
    if (!isDefined(arg3)) return ([
        [choke_action]
    ](arg1, arg2));
    return ([
        [choke_action]
    ](arg1, arg2, arg3));
}
_network_safe_spawn(classname, origin) {
    return Spawn(classname, origin);
}
network_choke_safe(id) {
    return (level.zombie_network_choke_ids_count[id] < level.zombie_network_choke_ids_max[id]);
}
tesla_play_death_fx() {
    tag = "J_SpineUpper";
    if (self.type == "dog") tag = "J_Spine1";
    fx = strTok("tesla_shock;tesla_shock_secondary", ";");
    playFxOnTag(level._effect[fx[randomInt(fx.size)]], self, tag);
    self playSound("imp_tesla");
    if (self.type != "dog") {
        if (randomInt(100) < level.zombie_vars["tesla_head_gib_chance"]) {
            wait(randomFloat(.53, 1));
            self zombie_head_gib();
        } else playFxOnTag(level._effect["tesla_shock_eyes"], self, "J_Eyeball_LE");
    }
}
treasure_chest_give_weapon(weapon_string) {
    primaryWeapons = self GetWeaponsListPrimaries();
    current_weapon = self getCurrentWeapon();
    if (self HasWeapon(weapon_string)) {
        self GiveMaxAmmo(weapon_string);
        self SwitchToWeapon(weapon_string);
        return;
    }
    if (primaryWeapons.size >= 2) {
        current_weapon = self getCurrentWeapon();
        if (current_weapon == "mine_bouncing_betty") current_weapon = undefined;
        if (isdefined(current_weapon)) {
            if (!(weapon_string == "fraggrenade" || weapon_string == "stielhandgranate" || weapon_string == "molotov" || weapon_string == "zombie_cymbal_monkey")) {
                if (isDefined(level.script) && (level.script == "nazi_zombie_sumpf" || level.script == "nazi_zombie_factory"))
                    if (current_weapon == "tesla_gun") level.player_drops_tesla_gun = true;
                self TakeWeapon(current_weapon);
            }
        }
    }
    if (IsDefined(primaryWeapons) && !isDefined(current_weapon)) {
        for (i = 0; i < primaryWeapons.size; i++) {
            if (primaryWeapons[i] == "zombie_colt") continue;
            if (weapon_string != "fraggrenade" && weapon_string != "stielhandgranate" && weapon_string != "molotov" && weapon_string != "zombie_cymbal_monkey") {
                if (isDefined(level.script) && (level.script == "nazi_zombie_sumpf" || level.script == "nazi_zombie_factory")) {
                    if (primaryWeapons[i] == "tesla_gun") level.player_drops_tesla_gun = true;
                }
                self TakeWeapon(primaryWeapons[i]);
            }
        }
    }
    self play_sound_on_ent("purchase");
    if (weapon_string == "molotov" || weapon_string == "molotov_zombie") {
        has_weapon = self HasWeapon("zombie_cymbal_monkey");
        if (isDefined(has_weapon) && has_weapon) self TakeWeapon("zombie_cymbal_monkey");
    }
    if (weapon_string == "zombie_cymbal_monkey") {
        has_weapon = self HasWeapon("molotov");
        if (isDefined(has_weapon) && has_weapon) self TakeWeapon("molotov");
        if (isDefined(level.zombie_weapons) && isDefined(level.zombie_weapons["molotov_zombie"])) {
            has_weapon = self HasWeapon("molotov_zombie");
            if (isDefined(has_weapon) && has_weapon) self TakeWeapon("molotov_zombie");
        }
        self player_give_cymbal_monkey();
        if (getMap() == "nzs" || getMap() == "nzf") play_weapon_vo(weapon_string);
        return;
    }
    self GiveWeapon(weapon_string, 0);
    self GiveMaxAmmo(weapon_string);
    self SwitchToWeapon(weapon_string);
    if (getMap() == "nzs" || getMap() == "nzf") play_weapon_vo(weapon_string);
}
play_weapon_vo(weapon) {
    index = get_player_index(self);
    if (!IsDefined(level.zombie_weapons[weapon].sound)) return;
    if (level.zombie_weapons[weapon].sound == "vox_monkey") {
        plr = "plr_" + index + "_";
        create_and_play_dialog(plr, "vox_monkey", .25, "resp_monk");
        return;
    }
    if (level.zombie_weapons[weapon].sound != "") {
        weap = level.zombie_weapons[weapon].sound;
        switch (weap) {
        case "vox_crappy":
            if (level.vox_crappy_available.size < 1) level.vox_crappy_available = level.vox_crappy;
            sound_to_play = random(level.vox_crappy_available);
            level.vox_crappy_available = array_delete(level.vox_crappy_available, sound_to_play);
            break;
        case "vox_mg":
            if (level.vox_mg_available.size < 1) level.vox_mg_available = level.vox_mg;
            sound_to_play = random(level.vox_mg_available);
            level.vox_mg_available = array_delete(level.vox_mg_available, sound_to_play);
            break;
        case "vox_shotgun":
            if (level.vox_shotgun_available.size < 1) level.vox_shotgun_available = level.vox_shotgun;
            sound_to_play = random(level.vox_shotgun_available);
            level.vox_shotgun_available = array_delete(level.vox_shotgun_available, sound_to_play);
            break;
        case "vox_357":
            if (level.vox_357_available.size < 1) level.vox_357_available = level.vox_357;
            sound_to_play = random(level.vox_357_available);
            level.vox_357_available = array_delete(level.vox_357_available, sound_to_play);
            break;
        case "vox_bar":
            if (level.vox_bar_available.size < 1) level.vox_bar_available = level.vox_bar;
            sound_to_play = random(level.vox_bar_available);
            level.vox_bar_available = array_delete(level.vox_bar_available, sound_to_play);
            break;
        case "vox_flame":
            if (level.vox_flame_available.size < 1) level.vox_flame_available = level.vox_flame;
            sound_to_play = random(level.vox_flame_available);
            level.vox_flame_available = array_delete(level.vox_flame_available, sound_to_play);
            break;
        case "vox_raygun":
            if (level.vox_raygun_available.size < 1) level.vox_raygun_available = level.vox_raygun;
            sound_to_play = random(level.vox_raygun_available);
            level.vox_raygun_available = array_delete(level.vox_raygun_available, sound_to_play);
            break;
        case "vox_tesla":
            if (level.vox_tesla_available.size < 1) level.vox_tesla_available = level.vox_tesla;
            sound_to_play = random(level.vox_tesla_available);
            level.vox_tesla_available = array_delete(level.vox_tesla_available, sound_to_play);
            break;
        case "vox_sticky":
            if (level.vox_sticky_available.size < 1) level.vox_sticky_available = level.vox_sticky;
            sound_to_play = random(level.vox_sticky_available);
            level.vox_sticky_available = array_delete(level.vox_sticky_available, sound_to_play);
            break;
        case "vox_ppsh":
            if (level.vox_ppsh_available.size < 1) level.vox_ppsh_available = level.vox_ppsh;
            sound_to_play = random(level.vox_ppsh_available);
            level.vox_ppsh_available = array_delete(level.vox_ppsh_available, sound_to_play);
            break;
        case "vox_mp40":
            if (level.vox_mp40_available.size < 1) level.vox_mp40_available = level.vox_mp40;
            sound_to_play = random(level.vox_mp40_available);
            level.vox_mp40_available = array_delete(level.vox_mp40_available, sound_to_play);
            break;
        default:
            sound_var = randomintrange(0, level.zombie_weapons[weapon].variation_count);
            sound_to_play = level.zombie_weapons[weapon].sound + "_" + sound_var;
        }
        plr = "plr_" + index + "_";
        self do_player_playdialog(plr, sound_to_play, 0.05);
    }
}
create_and_play_dialog(player_index, dialog_category, waittime, response) {
    if (!IsDefined(self.sound_dialog)) {
        self.sound_dialog = [];
        self.sound_dialog_available = [];
    }
    if (!IsDefined(self.sound_dialog[dialog_category])) {
        num_variants = get_number_variants(player_index + dialog_category);
        assertex(num_variants > 0, "No dialog variants found for category: " + dialog_category);
        for (i = 0; i < num_variants; i++) self.sound_dialog[dialog_category][i] = i;
        self.sound_dialog_available[dialog_category] = [];
    }
    if (self.sound_dialog_available[dialog_category].size <= 0) self.sound_dialog_available[dialog_category] = self.sound_dialog[dialog_category];
    variation = random(self.sound_dialog_available[dialog_category]);
    self.sound_dialog_available[dialog_category] = array_delete(self.sound_dialog_available[dialog_category], variation);
    sound_to_play = dialog_category + "_" + variation;
    self do_player_playdialog(player_index, sound_to_play, waittime, response);
}
do_player_playdialog(player_index, sound_to_play, waittime, response) {
    index = get_player_index(self);
    if (!IsDefined(level.player_is_speaking)) level.player_is_speaking = 0;
    if (level.player_is_speaking != 1) {
        level.player_is_speaking = 1;
        self playsound(player_index + sound_to_play, "sound_done" + sound_to_play);
        self waittill("sound_done" + sound_to_play);
        wait(waittime);
        level.player_is_speaking = 0;
        if (isdefined(response)) level thread setup_response_line(self, index, response);
    }
}
setup_response_line(player, index, response) {
    if (index == 0) setup_rival_hero(player, 1, 3, response);
    if (index == 1) setup_rival_hero(player, 3, 2, response);
    if (index == 2) setup_rival_hero(player, 0, 1, response);
    if (index == 3) setup_rival_hero(player, 2, 0, response);
    return;
}
setup_rival_hero(player, hero, rival, response) {
    players = getPlayers();
    playHero = isdefined(players[hero]);
    playRival = isdefined(players[rival]);
    if (playHero && playRival) {
        if (randomfloatrange(0, 1) < .5) playRival = false;
        else playHero = false;
    }
    if (playHero) {
        if (distancesquared(player.origin, players[hero].origin) < 500 * 500) {
            plr = "plr_" + hero + "_";
            players[hero] create_and_play_responses(plr, "vox_hr_" + response, 0.25);
        } else if (isdefined(players[rival])) playRival = true;
    }
    if (playRival) {
        if (distancesquared(player.origin, players[rival].origin) < 500 * 500) {
            plr = "plr_" + rival + "_";
            players[rival] create_and_play_responses(plr, "vox_riv_" + response, 0.25);
        }
    }
}
create_and_play_responses(player_index, dialog_category, waittime) {
    if (!IsDefined(self.sound_dialog)) {
        self.sound_dialog = [];
        self.sound_dialog_available = [];
    }
    if (!IsDefined(self.sound_dialog[dialog_category])) {
        num_variants = get_number_variants(player_index + dialog_category);
        assertex(num_variants > 0, "No dialog variants found for category: " + dialog_category);
        for (i = 0; i < num_variants; i++) self.sound_dialog[dialog_category][i] = i;
        self.sound_dialog_available[dialog_category] = [];
    }
    if (self.sound_dialog_available[dialog_category].size <= 0) self.sound_dialog_available[dialog_category] = self.sound_dialog[dialog_category];
    variation = random(self.sound_dialog_available[dialog_category]);
    self.sound_dialog_available[dialog_category] = array_delete(self.sound_dialog_available[dialog_category], variation);
    sound_to_play = dialog_category + "_" + variation;
    self do_player_playdialog(player_index, sound_to_play, waittime);
}
play_sound_on_ent(ref) {
    if (IsDefined(self.script_soundalias)) {
        self PlaySound(self.script_soundalias);
        return;
    }
    if (IsDefined(self.script_sound)) ref = self.script_sound;
    if (ref == "none") return;
    if (!IsDefined(level.zombie_sounds[ref])) {
        AssertMsg("Sound \"" + ref + "\" was not put to the zombie sounds list,please use add_sound(ref,alias)at the start of your level.");
        return;
    }
    self PlaySound(level.zombie_sounds[ref]);
}
find_flesh() {
    self endon("death");
    level endon("intermission");
    if (level.intermission) return;
    self.ignore_player = undefined;
    self.goalradius = 32;
    for (;;) {
        zombie_poi = self get_zombie_point_of_interest(self.origin);
        players = getPlayers();
        if (isDefined(level.zombieTheaterTeleporterSeekLogicFunc)) {
            temp_poi = self[[level.zombieTheaterTeleporterSeekLogicFunc]](players, zombie_poi);
            if (IsDefined(temp_poi)) zombie_poi = temp_poi;
        }
        if (players.size == 1) self.ignore_player = undefined;
        player = get_closest_valid_player(self.origin, self.ignore_player);
        if (!isDefined(player) && !isDefined(zombie_poi)) {
            if (IsDefined(self.ignore_player)) {
                self.ignore_player = undefined;
                self.ignore_player = [];
            }
            wait(1);
            continue;
        }
        self.ignore_player = undefined;
        self.enemyoverride = zombie_poi;
        self.favoriteenemy = player;
        self thread zombie_pathing();
        self.zombie_path_timer = GetTime() + (RandomFloatRange(1, 3) * 1000);
        while (GetTime() < self.zombie_path_timer) wait(0.1);
        self notify("zombie_acquire_enemy");
    }
}
get_zombie_point_of_interest(origin) {
    curr_radius = undefined;
    ent_array = getEntArray("zombie_poi", "script_noteworthy");
    best_poi = undefined;
    position = undefined;
    best_dist = 10000 * 10000;
    for (i = 0; i < ent_array.size; i++) {
        if (!isDefined(ent_array[i].poi_active) || !ent_array[i].poi_active) continue;
        dist = distanceSquared(origin, ent_array[i].origin);
        dist -= ent_array[i].added_poi_value;
        if (isDefined(ent_array[i].poi_radius)) curr_radius = ent_array[i].poi_radius;
        if ((!isDefined(curr_radius) || dist < curr_radius) && dist < best_dist && ent_array[i] can_attract(self)) best_poi = ent_array[i];
    }
    if (isDefined(best_poi)) {
        if (isDefined(best_poi.attract_to_origin) && best_poi.attract_to_origin) {
            position = [];
            position[0] = groundpos(best_poi.origin + (0, 0, 100));
            position[1] = self;
        } else position = self add_poi_attractor(best_poi);
    }
    return position;
}
get_closest_valid_player(origin, ignore_player) {
    valid_player_found = false;
    players = getPlayers();
    if (IsDefined(ignore_player)) players = array_delete(players, ignore_player);
    while (!valid_player_found) {
        player = GetClosest(origin, players);
        if (!isdefined(player)) return undefined;
        if (!is_player_valid(player)) {
            players = array_delete(players, player);
            continue;
        }
        return player;
    }
}
zombie_pathing() {
    self endon("death");
    self endon("zombie_acquire_enemy");
    level endon("intermission");
    assert(IsDefined(self.favoriteenemy) || isDefined(self.enemyoverride));
    self thread zombie_follow_enemy();
    self waittill("bad_path");
    if (isDefined(self.enemyoverride)) {
        if (isDefined(self.enemyoverride[1])) {
            self.enemyoverride = self.enemyoverride[1] invalidate_attractor_pos(self.enemyoverride, self);
            self.zombie_path_timer = 0;
            return;
        }
    }
    if (!isDefined(self.favoriteenemy)) {
        self.zombie_path_timer = 0;
        return;
    } else self.favoriteenemy endon("disconnect");
    crumb_list = self.favoriteenemy.zombie_breadcrumbs;
    bad_crumbs = [];
    for (;;) {
        if (!is_player_valid(self.favoriteenemy)) {
            self.zombie_path_timer = 0;
            return;
        }
        goal = zombie_pathing_get_breadcrumb(self.favoriteenemy.origin, crumb_list, bad_crumbs, (RandomInt(100) < 20));
        if (!IsDefined(goal)) goal = self.favoriteenemy.spectator_respawn.origin;
        self.zombie_path_timer += 1000;
        self SetGoalPos(goal);
        self waittill("bad_path");
        for (i = 0; i < crumb_list.size; i++) {
            if (goal == crumb_list[i]) {
                bad_crumbs[bad_crumbs.size] = i;
                break;
            }
        }
    }
}
zombie_follow_enemy() {
    self endon("death");
    self endon("zombie_acquire_enemy");
    self endon("bad_path");
    level endon("intermission");
    for (;;) {
        if (isDefined(self.enemyoverride) && isDefined(self.enemyoverride[1])) {
            if (distanceSquared(self.origin, self.enemyoverride[0]) > 1 * 1) {
                self OrientMode("face motion");
            } else {
                self OrientMode("face point", self.enemyoverride[1].origin);
            }
            self.ignoreall = true;
            self SetGoalPos(self.enemyoverride[0]);
        } else if (IsDefined(self.favoriteenemy)) {
            self.ignoreall = false;
            self OrientMode("face default");
            self SetGoalPos(self.favoriteenemy.origin);
        }
        if ((isDefined(level.script) && level.script == "nazi_zombie_sumpf")) {
            if ((isDefined(self.favoriteenemy.on_zipline) && self.favoriteenemy.on_zipline == true)) {
                crumb_list = self.favoriteenemy.zombie_breadcrumbs;
                bad_crumbs = [];
                goal = zombie_pathing_get_breadcrumb(self.favoriteenemy.origin, crumb_list, bad_crumbs, 0);
                if (isDefined(goal)) self SetGoalPos(goal);
            }
        }
        wait(0.1);
    }
}
invalidate_attractor_pos(attractor_pos, zombie) {
    if (!isDefined(self) || !isDefined(attractor_pos)) {
        wait(0.1);
        return undefined;
    }
    if (isDefined(self.attractor_positions) && !array_check_for_dupes(self.attractor_positions, attractor_pos)) {
        index = 0;
        for (i = 0; i < self.attractor_positions.size; i++)
            if (self.attractor_positions[i] == attractor_pos) index = i;
        for (i = 0; i < self.last_index.size; i++)
            if (index <= self.last_index[i]) self.last_index[i]--;
        self.attractor_array = array_delete(self.attractor_array, zombie);
        self.attractor_positions = array_delete(self.attractor_positions, attractor_pos);
        for (i = 0; i < self.claimed_attractor_positions.size; i++)
            if (self.claimed_attractor_positions[i][0] == attractor_pos[0]) self.claimed_attractor_positions = array_delete(self.claimed_attractor_positions, self.claimed_attractor_positions[i]);
    } else wait(0.1);
    return get_zombie_point_of_interest(zombie.origin);
}
zombie_pathing_get_breadcrumb(origin, breadcrumbs, bad_crumbs, pick_random) {
    assert(IsDefined(origin));
    assert(IsDefined(breadcrumbs));
    assert(IsArray(breadcrumbs));
    for (i = 0; i < breadcrumbs.size; i++) {
        if (pick_random) crumb_index = RandomInt(breadcrumbs.size);
        else crumb_index = i;
        if (crumb_is_bad(crumb_index, bad_crumbs)) continue;
        return breadcrumbs[crumb_index];
    }
    return undefined;
}
crumb_is_bad(crumb, bad_crumbs) {
    for (i = 0; i < bad_crumbs.size; i++)
        if (bad_crumbs[i] == crumb) return true;
    return false;
}
can_attract(attractor) {
    if (!isDefined(self.attractor_array)) self.attractor_array = [];
    if (!array_check_for_dupes(self.attractor_array, attractor)) return true;
    if (isDefined(self.num_poi_attracts) && self.attractor_array.size >= self.num_poi_attracts) return false;
    return true;
}
add_poi_attractor(zombie_poi) {
    if (!isDefined(zombie_poi)) return;
    if (!isDefined(zombie_poi.attractor_array)) zombie_poi.attractor_array = [];
    if (array_check_for_dupes(zombie_poi.attractor_array, self)) {
        if (!isDefined(zombie_poi.claimed_attractor_positions)) zombie_poi.claimed_attractor_positions = [];
        if (!isDefined(zombie_poi.attractor_positions) || zombie_poi.attractor_positions.size <= 0) return undefined;
        start = -1;
        end = -1;
        last_index = -1;
        for (i = 0; i < 4; i++) {
            if (zombie_poi.claimed_attractor_positions.size < zombie_poi.last_index[i]) {
                start = last_index + 1;
                end = zombie_poi.last_index[i];
                break;
            }
            last_index = zombie_poi.last_index[i];
        }
        best_dist = 10000 * 10000;
        best_pos = undefined;
        if (start < 0) start = 0;
        if (end < 0) return undefined;
        for (i = int(start); i <= int(end); i++) {
            if (array_check_for_dupes(zombie_poi.claimed_attractor_positions, zombie_poi.attractor_positions[i])) {
                dist = distancesquared(zombie_poi.attractor_positions[i][0], self.origin);
                if (dist < best_dist || !isDefined(best_pos)) {
                    best_dist = dist;
                    best_pos = zombie_poi.attractor_positions[i];
                }
            }
        }
        if (!isDefined(best_pos)) return undefined;
        zombie_poi.attractor_array = array_add(zombie_poi.attractor_array, self);
        self thread update_poi_on_death(zombie_poi);
        zombie_poi.claimed_attractor_positions = array_add(zombie_poi.claimed_attractor_positions, best_pos);
        return best_pos;
    } else
        for (i = 0; i < zombie_poi.attractor_array.size; i++)
            if (zombie_poi.attractor_array[i] == self)
                if (isDefined(zombie_poi.claimed_attractor_positions) && isDefined(zombie_poi.claimed_attractor_positions[i])) return zombie_poi.claimed_attractor_positions[i]; return undefined;
}
update_poi_on_death(zombie_poi) {
    self endon("kill_poi");
    self waittill("death");
    self remove_poi_attractor(zombie_poi);
}
remove_poi_attractor(zombie_poi) {
    if (!isDefined(zombie_poi.attractor_array)) return;
    for (i = 0; i < zombie_poi.attractor_array.size; i++) {
        if (zombie_poi.attractor_array[i] == self) {
            self notify("kill_poi");
            zombie_poi.attractor_array = array_delete(zombie_poi.attractor_array, zombie_poi.attractor_array[i]);
            zombie_poi.claimed_attractor_positions = array_delete(zombie_poi.claimed_attractor_positions, zombie_poi.claimed_attractor_positions[i]);
        }
    }
}
player_give_cymbal_monkey() {
    self giveweapon("zombie_cymbal_monkey");
    self thread player_handle_cymbal_monkey();
}
player_handle_cymbal_monkey() {
    self notify("starting_monkey_watch");
    self endon("disconnect");
    self endon("starting_monkey_watch");
    attract_dist_diff = level.monkey_attract_dist_diff;
    if (!isDefined(attract_dist_diff)) attract_dist_diff = 45;
    num_attractors = level.num_monkey_attractors;
    if (!isDefined(num_attractors)) num_attractors = 96;
    max_attract_dist = level.monkey_attract_dist;
    if (!isDefined(max_attract_dist)) max_attract_dist = 1536;
    while (true) {
        grenade = get_thrown_monkey();
        if (IsDefined(grenade)) {
            if (self maps\_laststand::player_is_in_laststand()) {
                grenade delete();
                continue;
            }
            grenade hide();
            model = spawn("script_model", grenade.origin);
            model setModel("weapon_zombie_monkey_bomb");
            model linkTo(grenade);
            model.angles = grenade.angles;
            grenade thread monitor_zombie_groans();
            velocitySq = 10000 * 10000;
            oldPos = grenade.origin;
            grenade create_zombie_point_of_interest(max_attract_dist, num_attractors, 10000);
            grenade.attract_to_origin = true;
            while (velocitySq != 0) {
                wait(0.05);
                velocitySq = distanceSquared(grenade.origin, oldPos);
                oldPos = grenade.origin;
            }
            if (isDefined(grenade)) {
                self achievement_notify("DLC3_ZOMBIE_USE_MONKEY");
                model thread monkey_cleanup(grenade);
                model unlink();
                model.origin = grenade.origin;
                model.angles = grenade.angles;
                grenade resetmissiledetonationtime();
                PlayFxOnTag(level._effect["monkey_glow"], model, "origin_animate_jnt");
                valid_poi = check_point_in_active_zone(grenade.origin);
                if (!valid_poi) valid_poi = check_point_in_playable_area(grenade.origin);
                if (valid_poi) {
                    grenade thread create_zombie_point_of_interest_attractor_positions(4, attract_dist_diff);
                    grenade thread wait_for_attractor_positions_complete();
                } else self.script_noteworthy = undefined;
                grenade thread do_monkey_sound(model, self);
            }
        }
        wait(0.05);
    }
}
wait_for_attractor_positions_complete() {
    self waittill("attractor_positions_generated");
    self.attract_to_origin = false;
}
monkey_cleanup(parent) {
    while (true) {
        if (!isDefined(parent)) {
            if (isDefined(self)) {
                self delete();
                return;
            }
        }
        wait(0.05);
    }
}
do_monkey_sound(model, player) {
    monk_scream_vox = false;
    if ((isdefined(level.monk_scream_trig)) && self IsTouching(level.monk_scream_trig)) {
        self playsound("monkey_scream_vox");
        monk_scream_vox = true;
    } else if (level.eggs == 0) {
        monk_scream_vox = false;
        self playsound("monkey_song");
    }
    wait(6.4);
    for (i = 0; i < self.sound_attractors.size; i++) {
        if (isDefined(self.sound_attractors[i])) self.sound_attractors[i] notify("monkey_blown_up");
    }
    if (!monk_scream_vox) self playsound("monkey_explo_vox");
    else if (monk_scream_vox) thread play_sam_furnace();
}
play_sam_furnace() {
    wait(2);
    play_sound_2d("sam_furnace_1");
    wait(2.5);
    play_sound_2d("sam_furnace_2");
}
get_thrown_monkey() {
    self endon("disconnect");
    self endon("starting_monkey_watch");
    while (true) {
        self waittill("grenade_fire", grenade, weapName);
        if (weapName == "zombie_cymbal_monkey") return grenade;
        wait(0.05);
    }
}
monitor_zombie_groans() {
    self.sound_attractors = [];
    self endon("monkey_blown_up");
    while (true) {
        if (!isDefined(self)) return;
        if (!isDefined(self.attractor_array)) {
            wait(0.05);
            continue;
        }
        for (i = 0; i < self.attractor_array.size; i++) {
            if (array_check_for_dupes(self.sound_attractors, self.attractor_array[i])) {
                if (distanceSquared(self.origin, self.attractor_array[i].origin) < 500 * 500) {
                    self.sound_attractors = array_add(self.sound_attractors, self.attractor_array[i]);
                    self.attractor_array[i] thread play_zombie_groans();
                }
            }
        }
        wait(0.05);
    }
}
play_zombie_groans() {
    self endon("death");
    self endon("monkey_blown_up");
    for (;;) {
        if (isdefined(self)) {
            self playsound("zombie_groan_monkey");
            wait randomfloatrange(2, 3);
        } else return;
    }
}
create_zombie_point_of_interest(attract_dist, num_attractors, added_poi_value, start_turned_on) {
    if (!isDefined(added_poi_value)) self.added_poi_value = 0;
    else self.added_poi_value = added_poi_value;
    if (!isDefined(start_turned_on)) start_turned_on = true;
    self.script_noteworthy = "zombie_poi";
    self.poi_active = start_turned_on;
    if (isDefined(attract_dist)) self.poi_radius = attract_dist * attract_dist;
    else self.poi_radius = undefined;
    self.num_poi_attracts = num_attractors;
    self.attract_to_origin = true;
}
create_zombie_point_of_interest_attractor_positions(num_attract_dists, diff_per_dist, attractor_width) {
    forward = (0, 1, 0);
    if (!isDefined(self.num_poi_attracts) || self.script_noteworthy != "zombie_poi") return;
    if (!isDefined(num_attract_dists)) num_attract_dists = 4;
    if (!isDefined(diff_per_dist)) diff_per_dist = 45;
    if (!isDefined(attractor_width)) attractor_width = 45;
    self.attract_to_origin = false;
    self.num_attract_dists = num_attract_dists;
    self.last_index = [];
    for (i = 0; i < num_attract_dists; i++) self.last_index[i] = -1;
    self.attract_dists = [];
    for (i = 0; i < self.num_attract_dists; i++) self.attract_dists[i] = diff_per_dist * (i + 1);
    max_positions = [];
    for (i = 0; i < self.num_attract_dists; i++) max_positions[i] = int(3.14 * 2 * self.attract_dists[i] / attractor_width);
    num_attracts_per_dist = self.num_poi_attracts / self.num_attract_dists;
    self.max_attractor_dist = self.attract_dists[self.attract_dists.size - 1] * 1.1;
    diff = 0;
    actual_num_positions = [];
    for (i = 0; i < self.num_attract_dists; i++) {
        if (num_attracts_per_dist > (max_positions[i] + diff)) {
            actual_num_positions[i] = max_positions[i];
            diff += num_attracts_per_dist - max_positions[i];
        } else {
            actual_num_positions[i] = num_attracts_per_dist + diff;
            diff = 0;
        }
    }
    self.attractor_positions = [];
    failed = 0;
    angle_offset = 0;
    prev_last_index = -1;
    for (j = 0; j < 4; j++) {
        if ((actual_num_positions[j] + failed) < max_positions[j]) {
            actual_num_positions[j] += failed;
            failed = 0;
        } else if (actual_num_positions[j] < max_positions[j]) {
            actual_num_positions[j] = max_positions[j];
            failed = max_positions[j] - actual_num_positions[j];
        }
        failed += self generated_radius_attract_positions(forward, angle_offset, actual_num_positions[j], self.attract_dists[j]);
        angle_offset += 15;
        self.last_index[j] = int(actual_num_positions[j] - failed + prev_last_index);
        prev_last_index = self.last_index[j];
    }
    self notify("attractor_positions_generated");
}
generated_radius_attract_positions(forward, offset, num_positions, attract_radius) {
    failed = 0;
    degs_per_pos = 360 / num_positions;
    for (i = offset; i < 360 + offset; i += degs_per_pos) {
        altforward = forward * attract_radius;
        rotated_forward = ((cos(i) * altforward[0] - sin(i) * altforward[1]), (sin(i) * altforward[0] + cos(i) * altforward[1]), altforward[2]);
        pos = server_safe_ground_trace("poi_trace", 10, self.origin + rotated_forward + (0, 0, 100));
        if (abs(pos[2] - self.origin[2]) < 60) {
            pos_array = [];
            pos_array[0] = pos;
            pos_array[1] = self;
            self.attractor_positions = array_add(self.attractor_positions, pos_array);
        } else failed++;
    }
    return failed;
}
check_point_in_playable_area(origin) {
    playable_area = getentarray("playable_area", "targetname");
    check_model = spawn("script_model", origin + (0, 0, 40));
    valid_point = false;
    for (i = 0; i < playable_area.size; i++)
        if (check_model istouching(playable_area[i])) valid_point = true;
    check_model delete();
    return valid_point;
}
check_point_in_active_zone(origin) {
    player_zones = GetEntArray("player_zone", "script_noteworthy");
    if (!isDefined(level.zones) || !isDefined(player_zones)) return true;
    scr_org = spawn("script_origin", origin + (0, 0, 40));
    one_valid_zone = false;
    for (i = 0; i < player_zones.size; i++)
        if (scr_org isTouching(player_zones[i]))
            if (isDefined(level.zones[player_zones[i].targetname]) && isDefined(level.zones[player_zones[i].targetname].is_enabled)) one_valid_zone = true;
    return one_valid_zone;
}
server_safe_ground_trace(id, max, origin) {
    server_safe_init(id, max);
    return (server_choke_action(id, ::_server_safe_ground_trace, origin));
}
server_safe_init(id, max) {
    if (!IsDefined(level.zombie_server_choke_ids_max) || !IsDefined(level.zombie_server_choke_ids_max[id])) server_choke_init(id, max);
    assert(max == level.zombie_server_choke_ids_max[id]);
}
server_choke_action(id, choke_action, arg1, arg2, arg3) {
    AssertEx(IsDefined(level.zombie_server_choke_ids_max[id]), "server Choke: " + id + " undefined");
    while (!server_choke_safe(id)) wait(0.05);
    level.zombie_server_choke_ids_count[id]++;
    if (!IsDefined(arg1)) return ([
        [choke_action]
    ]());
    if (!IsDefined(arg2)) return ([
        [choke_action]
    ](arg1));
    if (!IsDefined(arg3)) return ([
        [choke_action]
    ](arg1, arg2));
    return ([
        [choke_action]
    ](arg1, arg2, arg3));
}
_server_safe_ground_trace(pos) {
    return groundpos(pos);
}
server_choke_init(id, max) {
    if (!IsDefined(level.zombie_server_choke_ids_max)) {
        level.zombie_server_choke_ids_max = [];
        level.zombie_server_choke_ids_count = [];
    }
    level.zombie_server_choke_ids_max[id] = max;
    level.zombie_server_choke_ids_count[id] = 0;
    level thread server_choke_thread(id);
}
server_choke_thread(id) {
    for (;;) {
        wait(0.05);
        level.zombie_server_choke_ids_count[id] = 0;
    }
}
server_choke_safe(id) {
    return (level.zombie_server_choke_ids_count[id] < level.zombie_server_choke_ids_max[id]);
}
spectator_respawn() {
    println("*************************Respawn Spectator***");
    assert(IsDefined(self.spectator_respawn));
    origin = self.spectator_respawn.origin;
    angles = self.spectator_respawn.angles;
    self setSpectatePermissions(false);
    new_origin = undefined;
    new_origin = check_for_valid_spawn_near_team(self);
    if (IsDefined(new_origin)) self Spawn(new_origin, angles);
    else self Spawn(origin, angles);
    if (isSplitScreen()) {
        last_alive = undefined;
        players = getPlayers();
        for (i = 0; i < players.size; i++)
            if (!players[i].is_zombie) last_alive = players[i];
        share_screen(last_alive, false);
    }
    self.has_betties = undefined;
    self.is_burning = undefined;
    self.is_zombie = false;
    self.ignoreme = false;
    setClientSysState("lsm", "0", self);
    self RevivePlayer();
    self notify("spawned_player");
    self player_reduce_points("died");
    self thread player_zombie_breadcrumb();
    return true;
}
setSpectatePermissions(isOn) {
    self AllowSpectateTeam("allies", isOn);
    self AllowSpectateTeam("axis", false);
    self AllowSpectateTeam("freelook", false);
    self AllowSpectateTeam("none", false);
}
check_for_valid_spawn_near_team(revivee) {
    players = getPlayers();
    spawn_points = getstructarray("player_respawn_point", "targetname");
    if (spawn_points.size == 0) return undefined;
    for (i = 0; i < players.size; i++) {
        if (is_player_valid(players[i])) {
            for (j = 0; j < spawn_points.size; j++) {
                if (DistanceSquared(players[i].origin, spawn_points[j].origin) < (1000 * 1000) && spawn_points[j].locked == false) {
                    spawn_array = getstructarray(spawn_points[j].target, "targetname");
                    for (k = 0; k < spawn_array.size; k++) {
                        if (spawn_array[k].script_int == (revivee.entity_num + 1)) return spawn_array[k].origin;
                    }
                    return spawn_array[0].origin;
                }
            }
        }
    }
    return undefined;
}
player_reduce_points(event, mod, hit_location) {
    if (level.intermission) return;
    points = 0;
    switch (event) {
    case "no_revive_penalty":
        percent = level.zombie_vars["penalty_no_revive_percent"];
        points = self.score * percent;
        break;
    case "died":
        percent = level.zombie_vars["penalty_died_percent"];
        points = self.score * percent;
        break;
    case "downed":
        percent = level.zombie_vars["penalty_downed_percent"];;
        self notify("I_am_down");
        points = self.score * percent;
        self.score_lost_when_downed = round_up_to_ten(int(points));
        break;
    default:
        assertex(0, "Unknown point event");
        break;
    }
    points = self.score - round_up_to_ten(int(points));
    if (points < 0) points = 0;
    self.score = points;
    self set_player_score_hud();
}
round_up_to_ten(score) {
    new_score = score - score % 10;
    if (new_score < score) new_score += 10;
    return new_score;
}
player_zombie_breadcrumb() {
    self endon("disconnect");
    self endon("spawned_spectator");
    level endon("intermission");
    self.zombie_breadcrumbs = [];
    self.zombie_breadcrumb_distance = 24 * 24;
    self.zombie_breadcrumb_area_num = 3;
    self.zombie_breadcrumb_area_distance = 16;
    self store_crumb(self.origin);
    last_crumb = self.origin;
    for (;;) {
        wait_time = 0.1;
        store_crumb = true;
        airborne = false;
        crumb = self.origin;
        if (!self IsOnGround()) {
            airborne = true;
            store_crumb = false;
            wait_time = 0.05;
        }
        if (!airborne && DistanceSquared(crumb, last_crumb) < self.zombie_breadcrumb_distance) store_crumb = false;
        if (airborne && self IsOnGround()) {
            store_crumb = true;
            airborne = false;
        }
        if ((isDefined(level.script) && level.script == "nazi_zombie_sumpf" && (isDefined(self.on_zipline) && self.on_zipline == true))) {
            airborne = false;
            store_crumb = false;
        }
        if (store_crumb) {
            last_crumb = crumb;
            self store_crumb(crumb);
        }
        wait(wait_time);
    }
}
store_crumb(origin) {
    offsets = [];
    height_offset = 32;
    index = 0;
    for (j = 1; j <= self.zombie_breadcrumb_area_num; j++) {
        offset = (j * self.zombie_breadcrumb_area_distance);
        offsets[0] = (origin[0] - offset, origin[1], origin[2]);
        offsets[1] = (origin[0] + offset, origin[1], origin[2]);
        offsets[2] = (origin[0], origin[1] - offset, origin[2]);
        offsets[3] = (origin[0], origin[1] + offset, origin[2]);
        offsets[4] = (origin[0] - offset, origin[1], origin[2] + height_offset);
        offsets[5] = (origin[0] + offset, origin[1], origin[2] + height_offset);
        offsets[6] = (origin[0], origin[1] - offset, origin[2] + height_offset);
        offsets[7] = (origin[0], origin[1] + offset, origin[2] + height_offset);
        for (i = 0; i < offsets.size; i++) {
            self.zombie_breadcrumbs[index] = offsets[i];
            index++;
        }
    }
}
is_player_valid(player) {
    if (!IsDefined(player)) return false;
    if (!IsAlive(player)) return false;
    if (!IsPlayer(player)) return false;
    if (player.is_zombie == true) return false;
    if (player.sessionstate == "spectator") return false;
    if (player.sessionstate == "intermission") return false;
    if (player maps\_laststand::player_is_in_laststand()) return false;
    if (player isnotarget()) return false;
    return true;
}
chalk_one_up() {
    if (!IsDefined(level.doground_nomusic)) level.doground_nomusic = 0;
    if (level.first_round) intro = true;
    else intro = false;
    round = undefined;
    if (intro) {
        round = create_simple_hud();
        round.alignX = "center";
        round.alignY = "bottom";
        round.horzAlign = "center";
        round.vertAlign = "bottom";
        round.fontscale = 16;
        round.color = (1, 1, 1);
        round.x = 0;
        round.y = -265;
        round.alpha = 0;
        round setText("ZOMBIE_ROUND");
        round fadeOverTime(1);
        round.alpha = 1;
        wait(1);
        round fadeOverTime(3);
        round.color = (0.423, 0.004, 0);
    }
    hud = undefined;
    if (level.round_number < 6 || level.round_number > 10) {
        hud = level.chalk_hud1;
        hud.fontscale = 32;
    } else if (level.round_number < 11) hud = level.chalk_hud2;
    if (intro) {
        hud.alpha = 0;
        hud.horzAlign = "center";
        hud.x = -5;
        hud.y = -200;
    }
    hud fadeOverTime(0.5);
    hud.alpha = 0;
    if (level.round_number == 11 && IsDefined(level.chalk_hud2)) {
        level.chalk_hud2 fadeOverTime(0.5);
        level.chalk_hud2.alpha = 0;
    }
    wait(0.5);
    if (IsDefined(level.eggs) && level.eggs != 1) {
        if (level.doground_nomusic == 0) setMusicState("round_begin");
    }
    if (level.round_number == 11 && IsDefined(level.chalk_hud2)) level.chalk_hud2 destroy_hud();
    if (level.round_number > 10) hud setValue(level.round_number);
    hud fadeOverTime(0.5);
    hud.alpha = 1;
    if (intro) {
        wait(3);
        if (IsDefined(round)) {
            round fadeOverTime(1);
            round.alpha = 0;
        }
        wait(0.25);
        level notify("intro_hud_done");
        hud moveOverTime(1.75);
        hud.horzAlign = "left";
        hud.y = 0;
        wait(2);
        round destroy_hud();
    }
    if (level.round_number > 10) {} else if (level.round_number > 5) hud setShader("hud_chalk_" + (level.round_number - 5), 64, 64);
    else if (level.round_number > 1) hud setShader("hud_chalk_" + level.round_number, 64, 64);
}
create_simple_hud(client) {
    if (isDefined(client)) hud = newClientHudElem(client);
    else hud = newHudElem();
    level.hudelem_count++;
    hud.foreground = true;
    hud.sort = 1;
    hud.hidewheninmenu = false;
    return hud;
}
destroy_hud() {
    level.hudelem_count--;
    self destroy();
}
setMusicState(state) {
    if (level.musicState != state) setClientSysState("musicCmd", state);
    level.musicState = state;
}
zombie_head_gib(attacker) {
    if (is_german_build()) return;
    if (IsDefined(self.head_gibbed) && self.head_gibbed) return;
    if (!isDefined(level.head_gibs_this_frame)) level thread cap_zombie_head_gibs();
    if (level.head_gibs_this_frame >= level.max_head_gibs_per_frame) return;
    level.head_gibs_this_frame++;
    self.head_gibbed = true;
    size = self GetAttachSize();
    for (i = 0; i < size; i++) {
        model = self GetAttachModelName(i);
        if (IsSubStr(model, "head")) {
            self thread headshot_blood_fx();
            if (isdefined(self.hatmodel)) self detach(self.hatModel, "");
            self play_sound_on_ent("zombie_head_gib");
            self Detach(model, "", true);
            self Attach("char_ger_honorgd_zomb_behead", "", true);
            break;
        }
    }
    self thread damage_over_time(self.health * 0.2, 1, attacker);
}
cap_zombie_head_gibs() {
    if (!isDefined(level.max_head_gibs_per_frame)) level.max_head_gibs_per_frame = 4;
    while (true) {
        level.head_gibs_this_frame = 0;
        wait_network_frame();
    }
}
headshot_blood_fx() {
    if (!IsDefined(self)) return;
    if (!is_mature()) return;
    fxTag = "j_neck";
    fxOrigin = self GetTagOrigin(fxTag);
    upVec = AnglesToUp(self GetTagAngles(fxTag));
    forwardVec = AnglesToForward(self GetTagAngles(fxTag));
    PlayFX(level._effect["headshot"], fxOrigin, forwardVec, upVec);
    PlayFX(level._effect["headshot_nochunks"], fxOrigin, forwardVec, upVec);
    wait(0.3);
    if (IsDefined(self)) {
        if (self enemy_killed_by_tesla()) PlayFxOnTag(level._effect["tesla_head_light"], self, fxTag);
        else PlayFxOnTag(level._effect["bloodspurt"], self, fxTag);
    }
}
enemy_killed_by_tesla() {
    return (IsDefined(self.tesla_death) && self.tesla_death == true);
}
damage_over_time(dmg, delay, attacker) {
    self endon("death");
    if (!IsAlive(self)) return;
    if (!IsPlayer(attacker)) attacker = undefined;
    for (;;) {
        wait(delay);
        if (IsDefined(attacker)) self DoDamage(dmg, self.origin, attacker);
        else self DoDamage(dmg, self.origin);
    }
}
add_to_player_score(cost) {
    if (level.intermission) return;
    self.score += cost;
    self set_player_score_hud();
}
minus_to_player_score(cost) {
    if (level.intermission) return;
    self.score -= cost;
    self set_player_score_hud();
}
set_player_score_hud(init) {
    num = self.entity_num;
    score_diff = self.score - self.old_score;
    self thread score_highlight(self.score, score_diff);
    if (IsDefined(init)) return;
    self.old_score = self.score;
}
score_highlight(score, value) {
    self endon("disconnect");
    score_x = -103;
    score_y = -71;
    x = score_x;
    if (IsSplitScreen()) y = score_y;
    else {
        players = getPlayers();
        num = (players.size - self getEntityNumber()) - 1;
        y = (num * -18) + score_y;
    }
    time = 0.5;
    half_time = time * 0.5;
    hud = create_highlight_hud(x, y, value);
    hud moveOverTime(time);
    hud.x -= 20 + randomInt(40);
    hud.y -= (-15 + randomInt(30));
    wait(half_time);
    hud fadeOverTime(half_time);
    hud.alpha = 0;
    wait(half_time);
    hud destroy();
    level.hudelem_count--;
}
create_highlight_hud(x, y, value) {
    font_size = 8;
    if (isSplitScreen()) hud = NewClientHudElem(self);
    else hud = NewHudElem();
    level.hudelem_count++;
    hud.foreground = true;
    hud.sort = 0;
    hud.x = x;
    hud.y = y;
    hud.fontScale = font_size;
    hud.alignX = "right";
    hud.alignY = "middle";
    hud.horzAlign = "right";
    hud.vertAlign = "bottom";
    if (value < 1) hud.color = (0.423, 0.004, 0);
    else {
        hud.color = (0.9, 0.9, 0.0);
        hud.label = &"SCRIPT_PLUS";
    }
    hud.hidewheninmenu = false;
    hud setValue(value);
    return hud;
}
play_no_money_purchase_dialog() {
    index = get_player_index(self);
    player_index = "plr_" + index + "_";
    if (!IsDefined(self.vox_gen_sigh)) {
        num_variants = get_number_variants(player_index + "vox_gen_sigh");
        self.vox_gen_sigh = [];
        for (i = 0; i < num_variants; i++) self.vox_gen_sigh[self.vox_gen_sigh.size] = "vox_gen_sigh_" + i;
        self.vox_gen_sigh_available = self.vox_gen_sigh;
    }
    rand = randomintrange(0, 6);
    if (rand < 3) {
        sound_to_play = random(self.vox_gen_sigh_available);
        self.vox_gen_sigh_available = array_delete(self.vox_gen_sigh_available, sound_to_play);
        if (self.vox_gen_sigh_available.size < 1) self.vox_gen_sigh_available = self.vox_gen_sigh;
        wait 0.25;
        self do_player_playdialogx(player_index, sound_to_play, 0.25);
    }
}
do_player_playdialogx(player_index, sound_to_play, waittime) {
    if (!IsDefined(level.player_is_speaking)) level.player_is_speaking = 0;
    if (level.player_is_speaking != 1) {
        level.player_is_speaking = 1;
        self playsound(player_index + sound_to_play, "sound_done" + sound_to_play);
        self waittill("sound_done" + sound_to_play);
        wait(waittime);
        level.player_is_speaking = 0;
    }
}
get_player_index(player) {
    assert(IsPlayer(player));
    assert(IsDefined(player.entity_num));
    return player.entity_num;
}
get_number_variants(aliasPrefix) {
    for (i = 0; i < 100; i++)
        if (!SoundExists(aliasPrefix + "_" + i)) return i;
}
round_wait() {
    wait(1);
    if (flag("dog_round")) {
        wait(7);
        while (level.dog_intermission) wait(0.5);
    } else
        while (get_enemy_count() > 0 || level.zombie_total > 0 || level.intermission) wait(0.5);
}
get_enemy_count() {
    enemies = [];
    valid_enemies = [];
    enemies = GetAiSpeciesArray("axis", "all");
    for (i = 0; i < enemies.size; i++)
        if (enemies[i].animname != "boss_zombie") valid_enemies = array_add(valid_enemies, enemies[i]);
    return valid_enemies.size;
}
ai_calculate_health() {
    if (level.round_number >= 10) {
        level.zombie_health += int(level.zombie_health * level.zombie_vars["zombie_health_increase_percent"]);
        return;
    }
    if (level.round_number > 1) level.zombie_health = int(level.zombie_health + level.zombie_vars["zombie_health_increase"]);
}
zombie_setup_attack_properties() {
    self.ignoreall = false;
    self PushPlayer(true);
    self.pathEnemyFightDist = 64;
    self.meleeAttackDist = 64;
    self.maxsightdistsqrd = 128 * 128;
    self.disableArrivals = true;
    self.disableExits = true;
}
connect_zones(zone_name_a, zone_name_b, one_way) {
    if (!IsDefined(one_way)) one_way = false;
    zone_init(zone_name_a);
    zone_init(zone_name_b);
    enable_zone(zone_name_a);
    enable_zone(zone_name_b);
    if (!IsDefined(level.zones[zone_name_a].adjacent_zones[zone_name_b])) {
        level.zones[zone_name_a].adjacent_zones[zone_name_b] = spawnStruct();
        level.zones[zone_name_a].adjacent_zones[zone_name_b].is_connected = true;
    }
    if (!one_way) {
        if (!IsDefined(level.zones[zone_name_b].adjacent_zones[zone_name_a])) {
            level.zones[zone_name_b].adjacent_zones[zone_name_a] = SpawnStruct();
            level.zones[zone_name_b].adjacent_zones[zone_name_a].is_connected = true;
        }
    }
}
zone_init(zone_name) {
    if (isDefined(level.zones[zone_name])) return;
    level.zones[zone_name] = spawnstruct();
    level.zones[zone_name].is_enabled = false;
    level.zones[zone_name].is_occupied = false;
    level.zones[zone_name].is_active = false;
    level.zones[zone_name].adjacent_zones = [];
    level.zones[zone_name].volumes = GetEntArray(zone_name, "targetname");
    assertEx(isDefined(level.zones[zone_name].volumes[0]), "zone_init: No volumes found for zone:" + zone_name);
    if (IsDefined(level.zones[zone_name].volumes[0].target)) {
        level.zones[zone_name].spawners = [];
        level.zones[zone_name].dog_spawners = [];
        spawners = GetEntArray(level.zones[zone_name].volumes[0].target, "targetname");
        for (i = 0; i < spawners.size; i++) {
            if (issubstr(spawners[i].classname, "dog")) level.zones[zone_name].dog_spawners = add_to_array(level.zones[zone_name].dog_spawners, spawners[i]);
            else level.zones[zone_name].spawners = add_to_array(level.zones[zone_name].spawners, spawners[i]);
        }
        level.zones[zone_name].dog_locations = GetStructArray(level.zones[zone_name].volumes[0].target + "_dog", "targetname");
        level.zones[zone_name].rise_locations = GetStructArray(level.zones[zone_name].volumes[0].target + "_rise", "targetname");
    }
}
enable_zone(zone_name) {
    level.zones[zone_name].is_enabled = true;
    spawn_points = getstructarray("player_respawn_point", "targetname");
    for (i = 0; i < spawn_points.size; i++)
        if (spawn_points[i].script_noteworthy == zone_name) spawn_points[i].locked = false;
    activate_barrier_goals(zone_name + "_barriers", "script_noteworthy");
}
activate_barrier_goals(barrier_name, key) {
    entry_points = getstructarray(barrier_name, key);
    for (i = 0; i < entry_points.size; i++) {
        entry_points[i].is_active = 1;
        entry_points[i] trigger_on();
    }
}
zombie_setup_attack_properties_dog() {
    self thread dog_behind_audio();
    self.ignoreall = false;
    self.pathEnemyFightDist = 64;
    self.meleeAttackDist = 64;
    self.disableArrivals = true;
    self.disableExits = true;
}
dog_behind_audio() {
    self thread stop_dog_sound_on_death();
    self endon("death");
    self waittill_any("dog_running", "dog_combat");
    while (1) {
        wait(.25);
        players = get_players();
        for (i = 0; i < players.size; i++) {
            dogAngle = AngleClamp180(vectorToAngles(self.origin - players[i].origin)[1] - players[i].angles[1]);
            if (isAlive(players[i]) && !isDefined(players[i].revivetrigger)) {
                if ((abs(dogAngle) > 90) && distance2d(self.origin, players[i].origin) > 100) {
                    self playsound("zdog_close");
                }
            }
        }
    }
}
stop_dog_sound_on_death() {
    self waittill("death");
    self stopsounds();
}